﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Windows;

namespace ACDClientTest
{
    [ServiceContract]
    public interface IService
    {
        [OperationContract]
        void SetPosition( int count, int pos );
        [OperationContract]
        bool isAlive();
        [OperationContract]
        void Login();
    }

    public class Service : IService
    {
        public void SetPosition( int count, int pos )
        {
            try
            {
                if ( MainWindow.instance == null ) return;
                MainWindow.instance.SetPosition( count, pos );
            }
            catch ( Exception e )
            {
                Debug.WriteLine( e.Message );
            }
        }

        public bool isAlive()
        {
            return true;
        }

        public void Login()
        {
            try
            {
                if ( MainWindow.instance == null ) return;
                MainWindow.instance.ReLogin();
            }
            catch ( Exception e )
            {
                Debug.WriteLine( e.Message );
            }
        }
    }

    public class WindowServiceProxy : ClientBase<IService>
    {
        public WindowServiceProxy( string name )
            : base( new ServiceEndpoint( ContractDescription.GetContract( typeof( IService ) ),
                new NetNamedPipeBinding(), new EndpointAddress( "net.pipe://localhost/SoapTestService" + name + "/SoapTestService" ) ) )
        {

        }

        public void SetPosition( int count, int pos )
        {
            Channel.SetPosition( count, pos );
        }
        public bool isAlive()
        {
            return Channel.isAlive();
        }

        public void Login()
        {
            Channel.Login();
        }
    }

    public partial class App : Application
    {
        public const int MAX_INSTANCE = 3;
        public static ServiceHost GUIHost;
        private static int instanceNumber = -1;

        public static int InstanceNumber
        {
            get { return App.instanceNumber; }
            set { App.instanceNumber = value; }
        }

        public App()
        {

            GUIHost = TryRegisterPipe();
            if ( GUIHost == null ) return;

            Debug.WriteLine( "Service started. Available in following endpoints" );
            foreach ( var serviceEndpoint in GUIHost.Description.Endpoints )
            {
                Debug.WriteLine( serviceEndpoint.ListenUri.AbsoluteUri );
            }
        }

        private ServiceHost TryRegisterPipe()
        {
            ServiceHost host = null;
            for ( int i=0; i < MAX_INSTANCE; i++ )
            {
                try
                {
                    ServiceHost h = TryRegisterPipe( i.ToString() );
                    instanceNumber = i;
                    return h;
                }
                catch ( Exception e )
                {
                    Debug.WriteLine( e.Message );
                }
            }
            return host;
        }

        private static ServiceHost TryRegisterPipe( string name )
        {
            var serviceHost = new ServiceHost( typeof( Service ), new Uri[] { new Uri( "net.pipe://localhost/SoapTestService" + name ) } );
            serviceHost.AddServiceEndpoint( typeof( IService ), new NetNamedPipeBinding(), "SoapTestService" );
            serviceHost.Open();
            return serviceHost;
        }

        internal static List<WindowServiceProxy> GetWindowsProxies()
        {
            List<WindowServiceProxy> list  = new List<WindowServiceProxy>();
            for ( int i=0; i < MAX_INSTANCE; i++ )
            {
                try
                {
                    list.Add( TryGetProxy( i.ToString() ) );
                }
                catch ( Exception e )
                {
                    Debug.WriteLine( e.Message );
                }
            }
            return list;
        }

        private static WindowServiceProxy TryGetProxy( string name )
        {
            var p = new WindowServiceProxy( name );
            p.isAlive();
            return p;
        }
    }
}
