﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Net.Security;
using System.Windows.Threading;
using System.Text;

namespace ACDClientTest
{

    public partial class MainWindow : Window
    {
        public static MainWindow instance = null;
        Boolean pollingThreadQuit = true;
        Object pollingThreadSync = new Object();
        Thread eventThread = null;
        System.Collections.ArrayList comboBoxItemList = new System.Collections.ArrayList();
        private string sessionIDV3;

        public static bool OnValidationCallback( object sender, X509Certificate cert, X509Chain chain, SslPolicyErrors errors )
        {
            return true;
        }

        public MainWindow()
        {
            InitializeComponent();
            checkBoxAnnouncement.IsChecked = true;
            textBoxSessionUUID.IsReadOnly = true;
            BtnLogout.IsEnabled = false;

            if( !Properties.Settings.Default.ValidateCertificates )
            {
                ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback( OnValidationCallback );
            }
            instance = this;
        }

        public delegate void LogDelegate();

        private void logAdd( string log )
        {
            LogDelegate del = delegate
            {
                logWindow.AppendText( log + "\r\n" );
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        private void logClear()
        {
            LogDelegate del = delegate
            {
                logWindow.Clear();
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        private void eventLogItem( string log )
        {
            eventLogAdd( "    " + log, true );
        }

        private void eventLogAdd( string log )
        {
            eventLogAdd( log, false );
        }

        public int? getUsersIDParameter()
        {
            int? id = null;
            this.Dispatcher.Invoke( ( Action )( () =>
            {
                int result;
                id = Int32.TryParse( textBoxUsersID.Text, out result ) ? ( int? )result : null;
            } ) );
            return id;
        }

        private volatile bool lastOutputProgress = false;
        private void eventLogAdd( string log, bool withoutTime )
        {
            LogDelegate del = delegate
            {
                if ( log != "." && log != ":" )
                {
                    if ( !withoutTime )
                    {
                        string time = DateTime.Now.ToString( "HH:mm:ss.fff" );
                        eventWindow.AppendText( ( lastOutputProgress ? Environment.NewLine : "" ) + time + " " + log + Environment.NewLine );
                    }
                    else
                        eventWindow.AppendText( log + Environment.NewLine );
                    lastOutputProgress = false;
                }
                else
                {
                    lastOutputProgress = true;
                    eventWindow.AppendText( log );
                }
                if ( checkBoxAutoscroll.IsChecked.Value )
                {
                    eventWindow.ScrollToEnd();
                }
                Debug.WriteLine( log );
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        private void eventLogClear()
        {
            LogDelegate del = delegate
            {
                eventWindow.Clear();
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        private void setCallId( string callId )
        {
            LogDelegate del = delegate
            {
                textBoxCallID.Text = callId;
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        private void setButtonJunkCall( Boolean enable )
        {
            LogDelegate del = delegate
            {
                BtnJunkCall.IsEnabled = enable;
            };
            this.Dispatcher.BeginInvoke( del, System.Windows.Threading.DispatcherPriority.Send );
        }

        public delegate void ClickAction();

        private void SafeClick( ClickAction action )
        {
            try
            {
                action();
            }
            catch( Exception e )
            {
                logAdd( "error=" + e.Message );
            }
        }

        private void BtnLogin_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnLogin_ClickUnsafe( sender, e ) ); }
        private void BtnLogin_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            //variable to check if some field is empty
            bool someFieldIsNull = false;

            //clear log
            logClear();

            if( textBoxResellersUID.Text == "" && textBoxClientsUID.Text != "" )
            {
                logAdd( "Reseller UID required" );
                someFieldIsNull = true;
            }
            else if( textBoxUsersUID.Text == "" )
            {
                logAdd( "User UID required" );
                someFieldIsNull = true;
            }
            else if( !someFieldIsNull && passwordBoxPassword.Password == "" )
            {
                logAdd( "Password required" );
                someFieldIsNull = true;
            }

            //any field is null
            if( !someFieldIsNull )
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.Language lang;
                ACDWebServiceV3.Language[] langs = null;
                sessionIDV3 = "";
                resultV3 = wsClient2.login( textBoxResellersUID.Text, textBoxClientsUID.Text, textBoxUsersUID.Text, passwordBoxPassword.Password, out sessionIDV3, out lang, out langs );
                logAdd( "Login ()=" + resultV3.ToString() );
                if( langs == null || langs.Length == 0 )
                    logAdd( "There is no supported languages" );
                else
                    for( int i = 0; i < langs.Length; i++ )
                    {
                        logAdd( "[" + langs[ i ].id + "] isoCode = " + langs[ i ].isoCode + ", shortISOCode = " + langs[ i ].shortISOCode );
                        logAdd( "   name = " + langs[ i ].name );
                    }

                if( sessionIDV3 != null )
                {
                    logAdd( "Session ID = " + sessionIDV3 );
                    logAdd( "Starting event thread..." );

                    // Start event thread
                    startEventThread( sessionIDV3 );

                    BtnLogin.IsEnabled = false;

                    textBoxResellersUID.IsEnabled = false;
                    textBoxClientsUID.IsEnabled = false;
                    textBoxUsersUID.IsEnabled = false;
                    passwordBoxPassword.IsEnabled = false;

                    BtnLogout.IsEnabled = true;

                    ACDWebServiceV3.AcdAccount accountData = GetAccountData( null );
                    textBoxUsersID.Background =
                            ( accountData != null && accountData.bSuperuser.HasValue && accountData.bSuperuser.Value )
                            ? Brushes.Bisque
                            : Brushes.Gray;
                    textBoxUsersID.Text = "? " + accountData.userId;
                }

                //set field sessionID
                textBoxSessionUUID.Text = sessionIDV3;
            }
        }

        private void doLogout()
        {
            //check if a user is logged in
            if( ( sessionIDV3 == null ) || ( sessionIDV3 == "" ) )
                logAdd( "Not logged in" );
            else
            {
                try
                {
                    ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client();
                    ACDClientTest.ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                    resultV3 = wsClient2.logout( sessionIDV3 );
                    sessionIDV3 = "";

                    logAdd( "Logout()=" + resultV3.ToString() );
                }
                catch( Exception e )
                {
                    logAdd( "Logout FAILED: " + e.Message );
                }

                //clear field
                textBoxSessionUUID.Clear();

                // stop event thread
                stopEventThread();

                BtnLogin.IsEnabled = true;
                BtnLogout.IsEnabled = false;

                //clear fields
                setDefaultValuesAndClearAllFields();

                textBoxResellersUID.IsEnabled = true;
                textBoxClientsUID.IsEnabled = true;
                textBoxUsersUID.IsEnabled = true;
                passwordBoxPassword.IsEnabled = true;
                textBoxUsersID.Background = Brushes.White;
            }
        }

        private void BtnLogout_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnLogout_ClickUnsafe( sender, e ) ); }
        private void BtnLogout_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            doLogout();
        }

        private void BtnGetAccountData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetAccountData_ClickUnsafe( sender, e ) ); }
        private void BtnGetAccountData_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            GetAccountData( getUsersIDParameter() );
        }
        private ACDWebServiceV3.AcdAccount GetAccountData( int? usersID )
        {
            ACDWebServiceV3.AcdAccount accountData = null;
            //check if a user is logged in
            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClientV3 = new ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClientV3.getAccountData( sessionIDV3, usersID, out accountData );

                logAdd( "GetAccountData()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( accountData != null )
                    logAdd(
                        "User Id: " + accountData.userId
                        + "\nName: " + accountData.firstName
                        + "\nLast Name: " + accountData.lastName
                        + "\nAgent Number: " + accountData.agentNumber
                        + "\nEmail: " + accountData.email
                        + "\nClient ID: " + accountData.clientId
                        + "\nClient Name: " + accountData.clientName
                        + "\nReseller Id: " + accountData.resellerId
                        + "\nReseller Name: " + accountData.resellerName
                        + "\nPhone Number: " + accountData.phoneNumber
                        + "\nPhone Number1: " + accountData.phoneNumber1
                        + "\nPhone Number2: " + accountData.phoneNumber2
                        + "\nPhone Number3: " + accountData.phoneNumber3
                        + "\nPhone Number4: " + accountData.phoneNumber4
                        + "\nPhone Number5: " + accountData.phoneNumber5
                        + "\nPhone Number6: " + accountData.phoneNumber6
                        + "\nbFreeSeating: " + accountData.bFreeSeating
                        + "\nbFreeSeatingWhitelist: " + accountData.bFreeSeatingWhitelist
                        + "\nbSMSActive: " + accountData.bSMSActive
                        + "\nbPBXConnector: " + accountData.bPBXConnector
                        + "\nbCanSetAnyCallerNumber: " + accountData.bCanSetAnyCallerNumber
                        + "\nbSuperuser: " + accountData.bSuperuser
                        );
            }
            return accountData;
        }

        private string getUsersIDParameterInfo()
        {
            int? id = getUsersIDParameter();
            return ", usersID=" + ( id.HasValue ? id.Value.ToString() : "null" );
        }

        private void BtnGetAgentGroupsData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetAgentGroupsData_ClickUnsafe( sender, e ) ); }
        private void BtnGetAgentGroupsData_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            //check if a user is logged in
            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AcdGroup[] acdGroups;

                resultV3 = wsClient2.getAgentGroupsData( sessionIDV3, getUsersIDParameter(), out acdGroups );
                comboAcdGroups.Items.Clear();

                logAdd( "GetAgentGroupsData()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( acdGroups != null )
                {
                    if( acdGroups.Length == 0 )
                        logAdd( "There is no ACD Group for this Agent" );
                    else
                        comboAcdGroups.Items.Clear();
                    for( int i = 0; i < acdGroups.Length; i++ )
                    {
                        PrintGroupData( acdGroups[ i ] );
                        ComboBoxItem item = new ComboBoxItem();
                        item.Content = acdGroups[ i ].name;
                        item.Tag = acdGroups[ i ].id;
                        comboAcdGroups.Items.Add( item );
                    }
                }
            }
        }

        private void BtnGetAgentOutboundGroupsData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetAgentOutboundGroupsData_ClickUsafe( sender, e ) ); }
        private void BtnGetAgentOutboundGroupsData_ClickUsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            //check if a user is logged in
            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AcdGroup[] acdGroups;

                resultV3 = wsClient2.getAgentOutboundGroupsData( sessionIDV3, getUsersIDParameter(), out acdGroups );
                comboAcdGroups.Items.Clear();

                logAdd( "GetAgentGroupsData()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( acdGroups != null )
                {
                    if( acdGroups.Length == 0 )
                        logAdd( "There is no ACD Group for this Agent" );
                    else
                        comboAcdGroups.Items.Clear();
                    for( int i = 0; i < acdGroups.Length; i++ )
                    {
                        PrintGroupData( acdGroups[ i ] );
                        ComboBoxItem item = new ComboBoxItem();
                        item.Content = acdGroups[ i ].name;
                        item.Tag = acdGroups[ i ].id;
                        comboAcdGroups.Items.Add( item );
                    }
                }
            }

        }

        private void PrintGroupData( ACDWebServiceV3.AcdGroup acdGroups )
        {
            logAdd( "Group ID: " + acdGroups.id + ", Name: " + acdGroups.name );
            logAdd( "     agentCount: " + acdGroups.agentCount + ", agentSkill: " + acdGroups.agentSkill );
            logAdd( "     dtLoginLogout: " + acdGroups.dtLoginLogout + ", groupNumber: " + acdGroups.groupNumber );
            logAdd( "     groupPriority: " + acdGroups.groupPriority + ", loggedIn: " + acdGroups.loggedIn );
            logAdd( "     groupType: " + acdGroups.groupType );
        }

        private void BtnGetAllGroupsData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetAllGroupsData_ClickUsafe( sender, e ) ); }
        private void BtnGetAllGroupsData_ClickUsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            //check if a user is logged in
            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AcdGroup[] acdGroups;

                resultV3 = wsClient2.getClientGroupsData( sessionIDV3, getUsersIDParameter(), out acdGroups );
                logAdd( "getClientGroupsData()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( acdGroups != null )
                {
                    if( acdGroups.Length == 0 )
                        logAdd( "There is no ACD Group" );
                    else
                        comboBoxClickToCallACDGroup.Items.Clear();
                    comboBoxATransferACDGroup.Items.Clear();
                    for( int i = 0; i < acdGroups.Length; i++ )
                    {
                        PrintGroupData( acdGroups[ i ] );
                        ComboBoxItem item = new ComboBoxItem();
                        item.Content = acdGroups[ i ].name;
                        item.Tag = acdGroups[ i ].id;
                        comboBoxATransferACDGroup.Items.Add( item );
                        item = new ComboBoxItem();
                        item.Content = acdGroups[ i ].name;
                        item.Tag = acdGroups[ i ].id;
                        comboBoxClickToCallACDGroup.Items.Add( item );
                        item = new ComboBoxItem();
                        item.Content = acdGroups[ i ].name;
                        item.Tag = acdGroups[ i ].id;
                        comboAcdGroupsAdmin.Items.Add( item );
					}
                }
            }
        }

        private void BtnGetAgentStatus_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetAgentStatus_ClickUnsafe( sender, e ) ); }
        private void BtnGetAgentStatus_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AgentStatus agentStatus;

                resultV3 = wsClient2.getAgentStatus( sessionIDV3, getUsersIDParameter(), out agentStatus );

                logAdd( "GetAgentStatus()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( agentStatus != null )
                {

                    logAdd( "acdAgentStatusID: " + agentStatus.acdAgentStatusID );
                    logAdd( "acdTelephoneNumber: " + agentStatus.acdTelephoneNumber );
                    logAdd( "acdTelephoneStatus: " + agentStatus.acdTelephoneStatus );
                    logAdd( "dtAcdAgentStatus: " + agentStatus.dtAcdAgentStatus );
                    logAdd( "dtNextCall: " + agentStatus.dtNextCall );
                    logAdd( "dtPbxTelephoneStatus: " + agentStatus.dtPbxTelephoneStatus );
                    logAdd( "dtTelephoneStatus: " + agentStatus.dtTelephoneStatus );
                    logAdd( "nMediaEventsNew: " + agentStatus.nMediaEventsNew );
                    logAdd( "nMediaEventsSeen: " + agentStatus.nMediaEventsSeen );
                    logAdd( "nWebMessagesNew: " + agentStatus.nWebMessagesNew );
                    logAdd( "nWebMessagesSeen: " + agentStatus.nWebMessagesSeen );
                    logAdd( "pbxTelephoneStatus: " + agentStatus.pbxTelephoneStatus );

                    textBoxCurrentStatusAgent.Text = agentStatus.acdAgentStatusID.ToString();
                    populatedComboBoxListStatusACD( agentStatus.acdAgentStatusID );
                }
            }
        }

        private void BtnCallHangUp_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallHangUp_ClickUnsafe( sender, e ) ); }
        private void BtnCallHangUp_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textBoxCallID.Text != "" )
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.callHangUp( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

                logAdd( "CallHangUp()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "Call ID=" + textBoxCallID.Text );
            }
            else // call ID is null
                logAdd( "Call ID required" );
        }

        private void BtnCallTransferNumber_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferNumber_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferNumber_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            // variable to check if phone is a number
            bool phoneIsNumber = System.Text.RegularExpressions.Regex.IsMatch( textBoxPhoneNumberTransfer.Text, "^[0-9]+$" );

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                bool announcement = checkBoxAnnouncement.IsChecked.HasValue && ( checkBoxAnnouncement.IsChecked == true );
                resultV3 = wsClient2.callTransferNumber( sessionIDV3, textBoxCallID.Text, textBoxPhoneNumberTransfer.Text, getUsersIDParameter() );

                // add log
                logClear();
                logAdd( "Call ID=" + textBoxCallID.Text );
                logAdd( "Phone Number to transfer=" + textBoxPhoneNumberTransfer.Text );
                logAdd( "CallTransferNumber()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }
        }

        private void BtnCallTransferSynonym_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferSynonym_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferSynonym_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else if( textBoxCallID.Text == "" )
                logAdd( "Call ID required" );
            else if( comboBoxTransferSynonym.SelectedItem == null )
                logAdd( "Select a Synonym" );
            else
            {
                ComboBoxItem item = ( ComboBoxItem ) comboBoxTransferSynonym.SelectedItem;
                int synonymID = ( Int32 ) item.Tag;
                String synonymName = ( String ) item.Content;
                int foundSynonymID;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                //call transfer synonym
                bool announcement = checkBoxAnnouncement.IsChecked.HasValue && ( checkBoxAnnouncement.IsChecked == true );

                resultV3 = wsClient2.callTransferSynonym( sessionIDV3, textBoxCallID.Text, synonymID, synonymName, announcement, getUsersIDParameter(), out foundSynonymID );

                logAdd( "ID Call=" + textBoxCallID.Text );
                logAdd( "Synonym ID=" + synonymID );
                logAdd( "Synonym Name=" + synonymName );
                logAdd( "Announcement=" + checkBoxAnnouncement.ToString() );
                logAdd( "CallTransferSynonym()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "Found Synonym ID=" + foundSynonymID );
            }
        }

        private void BtnCallTransferACDGroup_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferACDGroup_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferACDGroup_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textBoxCallID.Text == "" )
                logAdd( "Call ID required" );
            else if( comboBoxATransferACDGroup.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                ComboBoxItem item = ( ComboBoxItem ) comboBoxATransferACDGroup.SelectedItem;
                Int32 AcdGroupsID = ( Int32 ) item.Tag;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.callTransferAcdGroup( sessionIDV3, textBoxCallID.Text, AcdGroupsID, getUsersIDParameter() );

                logAdd( "ID Call=" + textBoxCallID.Text );
                logAdd( "Acd Groups Name=" + ( String )item.Content );
                logAdd( "Acd Groups ID=" + AcdGroupsID.ToString() );
                logAdd( "CallTransferAcdGroup()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }

        }

        private void BtnChangeStatus_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnChangeStatus_ClickUnsafe( sender, e ) ); }
        private void BtnChangeStatus_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            bool phoneIsANumber = System.Text.RegularExpressions.Regex.IsMatch( textBoxPhoneNumberAgentStatus.Text, "^[0-9]+$" );

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboBoxNewStatusIDAgent.SelectedItem == null )
                logAdd( "Select a new ACD status" );
            //check if phone is a number and conversion was successful
            else if( textBoxPhoneNumberAgentStatus.Text != "" && !phoneIsANumber )
                logAdd( "Phone Number must be a number. \n" );
            else //conversion was successful and phone is a number
            {
                ComboBoxItem item = ( ComboBoxItem ) comboBoxNewStatusIDAgent.SelectedItem;
                Int32 NewStatusID = ( Int32 ) item.Tag;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.setAgentStatus( sessionIDV3, NewStatusID, textBoxPhoneNumberAgentStatus.Text, textBoxComment.Text, getUsersIDParameter() );

                logAdd( "ChangeStatus()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( textBoxPhoneNumberAgentStatus.Text != "" ) logAdd( "Phone Number=" + textBoxPhoneNumberAgentStatus.Text );
                logAdd( "New ID Status=" + NewStatusID );
                logAdd( "New Name Status=" + getNameACDStatus( NewStatusID ) );
                if( textBoxComment.Text != "" ) logAdd( "Comments=" + textBoxComment.Text );

                textBoxComment.Clear();

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    populatedComboBoxListStatusACD( NewStatusID );
                    textBoxCurrentStatusAgent.Text = getNameACDStatus( NewStatusID );
                }
                else
                    comboBoxNewStatusIDAgent.SelectedItem = null;
            }
        }

        private void BtnExtendPostCall_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnExtendPostCall_ClickUnsafe( sender, e ) ); }
        private void BtnExtendPostCall_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            int Interval;
            //variable to check if Interval is a number
            bool intervalIsANumber= Int32.TryParse( textBoxInterval.Text, out Interval );

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            //check if Interval is a number
            else if( !intervalIsANumber )
                logAdd( "Interval must be a number" );
            else //Interval is not a number
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.extendPostCall( sessionIDV3, Interval, getUsersIDParameter() );

                logAdd( "ExtendPostCall()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "Interval=" + textBoxInterval.Text );
            }
            textBoxInterval.Clear();
        }

        private void BtnGetNextEvent_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetNextEvent_ClickUnsafe( sender, e ) ); }
        private void BtnGetNextEvent_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                String stateData = "";
                ACDWebServiceV3.AcdEvent[] eventData;
                resultV3 = wsClient2.getNextEvents( sessionIDV3, ref stateData, out eventData );
                logAdd( "getNextEvents (" + sessionIDV3 + ")=" + resultV3.ToString() );
                logEventData( eventData );
            }
        }

        private void logEventData( ACDWebServiceV3.AcdEvent[] eventData )
        {
            eventLogItem( "eventData.Length=" + eventData.Length );
            int i = 0;
            foreach( ACDWebServiceV3.AcdEvent acdEvent in eventData )
            {
                eventLogItem( "eventType=" + acdEvent.eventType.ToString() + " dtServer=" + acdEvent.dtEventTimestamp.ToString( "HH:mm:ss.fff" ) );

                if( acdEvent.eventInformation.callIncomingData != null )
                {
                    eventLogItem( "[" + i + "]callIncomingData.userID=" + acdEvent.eventInformation.callIncomingData.userID );
                    eventLogItem( "[" + i + "]callIncomingData.acdGroupID=" + acdEvent.eventInformation.callIncomingData.acdGroupID );
                    eventLogItem( "[" + i + "]callIncomingData.acdGroupName=" + acdEvent.eventInformation.callIncomingData.acdGroupName );
                    eventLogItem( "[" + i + "]callIncomingData.additionalInfo=" + acdEvent.eventInformation.callIncomingData.additionalInfo );
                    eventLogItem( "[" + i + "]callIncomingData.bCanSetJunk=" + acdEvent.eventInformation.callIncomingData.bCanSetJunk );
                    eventLogItem( "[" + i + "]callIncomingData.bMandatoryTransactionCode=" + acdEvent.eventInformation.callIncomingData.bMandatoryTransactionCode );
                    eventLogItem( "[" + i + "]callIncomingData.bOneTransactionCodeOnly=" + acdEvent.eventInformation.callIncomingData.bOneTransactionCodeOnly );
                    eventLogItem( "[" + i + "]callIncomingData.callerNumber=" + acdEvent.eventInformation.callIncomingData.callerNumber );
                    eventLogItem( "[" + i + "]callIncomingData.callRecordingType=" + acdEvent.eventInformation.callIncomingData.callRecordingType );
                    eventLogItem( "[" + i + "]callIncomingData.defaultTransactionCodesID=" + acdEvent.eventInformation.callIncomingData.defaultTransactionCodesID );
                    eventLogItem( "[" + i + "]callIncomingData.dtAcdDistributeStart=" + acdEvent.eventInformation.callIncomingData.dtAcdDistributeStart );
                    eventLogItem( "[" + i + "]callIncomingData.dtAcdGroupStart=" + acdEvent.eventInformation.callIncomingData.dtAcdGroupStart );
                    eventLogItem( "[" + i + "]callIncomingData.dtCallStart=" + acdEvent.eventInformation.callIncomingData.dtCallStart );
                    eventLogItem( "[" + i + "]callIncomingData.dtAcdQueueStart=" + acdEvent.eventInformation.callIncomingData.dtAcdQueueStart );
                    eventLogItem( "[" + i + "]callIncomingData.callID=" + acdEvent.eventInformation.callIncomingData.callID );
                    eventLogItem( "[" + i + "]callIncomingData.serviceNumber=" + acdEvent.eventInformation.callIncomingData.serviceNumber );
                    eventLogItem( "[" + i + "]callIncomingData.referringAgentID=" + acdEvent.eventInformation.callIncomingData.referringAgentID );
					eventLogItem( "[" + i + "]callIncomingData.languageID=" + acdEvent.eventInformation.callIncomingData.languageID );
					eventLogItem( "[" + i + "]callIncomingData.userData=" + acdEvent.eventInformation.callIncomingData.userData );
					eventLogItem( "[" + i + "]callIncomingData.statisticsPartAID=" + acdEvent.eventInformation.callIncomingData.statisticsPartAID );
					eventLogItem( "[" + i + "]callIncomingData.testCall=" + acdEvent.eventInformation.callIncomingData.testCall );
				}
				if( acdEvent.eventInformation.callConnectedData != null )
                {
                    eventLogItem( "[" + i + "]callConnectedData.userID=" + acdEvent.eventInformation.callConnectedData.userID );
                    eventLogItem( "[" + i + "]callConnectedData.callID=" + acdEvent.eventInformation.callConnectedData.callID );
					eventLogItem( "[" + i + "]callConnectedData.testCall=" + acdEvent.eventInformation.callConnectedData.testCall );
                    setCallId( acdEvent.eventInformation.callConnectedData.callID );
                }
                if( acdEvent.eventInformation.callEndData != null )
                {
                    eventLogItem( "[" + i + "]callEndData.userID=" + acdEvent.eventInformation.callEndData.userID );
                    eventLogItem( "[" + i + "]callEndData.callInBackground=" + acdEvent.eventInformation.callEndData.callInBackground );
                    eventLogItem( "[" + i + "]callEndData.maxPostCallInterval=" + acdEvent.eventInformation.callEndData.maxPostCallInterval );
                    eventLogItem( "[" + i + "]callEndData.postCallInterval=" + acdEvent.eventInformation.callEndData.postCallInterval );
                    eventLogItem( "[" + i + "]callEndData.postCallIntervalAdd=" + acdEvent.eventInformation.callEndData.postCallIntervalAdd );
                    eventLogItem( "[" + i + "]callEndData.callID=" + acdEvent.eventInformation.callEndData.callID );
                    eventLogItem( "[" + i + "]callEndData.result=" + acdEvent.eventInformation.callEndData.result );
					eventLogItem( "[" + i + "]callEndData.testCall=" + acdEvent.eventInformation.callEndData.testCall );
                }
                if( acdEvent.eventInformation.callOutgoingAgentConnectedData != null )
                {
                    eventLogItem( "[" + i + "]callOutgoingAgentConnectedData.userID=" + acdEvent.eventInformation.callOutgoingAgentConnectedData.userID );
                    eventLogItem( "[" + i + "]callOutgoingAgentConnectedData.callID=" + acdEvent.eventInformation.callOutgoingAgentConnectedData.callID );
					eventLogItem( "[" + i + "]callOutgoingAgentConnectedData.testCall=" + acdEvent.eventInformation.callOutgoingAgentConnectedData.testCall );
                }
                if( acdEvent.eventInformation.callOutgoingAgentData != null )
                {
                    eventLogItem( "[" + i + "]callOutgoingAgentData.userID=" + acdEvent.eventInformation.callOutgoingAgentData.userID );
                    eventLogItem( "[" + i + "]callOutgoingAgentData.callID=" + acdEvent.eventInformation.callOutgoingAgentData.callID );
					eventLogItem( "[" + i + "]callOutgoingAgentData.testCall=" + acdEvent.eventInformation.callOutgoingAgentData.testCall );
                }
                if( acdEvent.eventInformation.callOutgoingAgentEndData != null )
                {
                    eventLogItem( "[" + i + "]callOutgoingAgentEndData.userID=" + acdEvent.eventInformation.callOutgoingAgentEndData.userID );
                    eventLogItem( "[" + i + "]callOutgoingAgentEndData.callID=" + acdEvent.eventInformation.callOutgoingAgentEndData.callID );
                    eventLogItem( "[" + i + "]callOutgoingAgentEndData.result=" + acdEvent.eventInformation.callOutgoingAgentEndData.result );
					eventLogItem( "[" + i + "]callOutgoingAgentEndData.testCall=" + acdEvent.eventInformation.callOutgoingAgentEndData.testCall );
                }
                if( acdEvent.eventInformation.callOutgoingDestinationData != null )
                {
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.userID=" + acdEvent.eventInformation.callOutgoingDestinationData.userID );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.callID=" + acdEvent.eventInformation.callOutgoingDestinationData.callID );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.acdConfigurationGroupID=" + acdEvent.eventInformation.callOutgoingDestinationData.acdConfigurationGroupID );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.acdConfigurationGroupName=" + acdEvent.eventInformation.callOutgoingDestinationData.acdConfigurationGroupName );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.acdGroupID=" + acdEvent.eventInformation.callOutgoingDestinationData.acdGroupID );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.acdGroupName=" + acdEvent.eventInformation.callOutgoingDestinationData.acdGroupName );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.bMandatoryTransactionCode=" + acdEvent.eventInformation.callOutgoingDestinationData.bMandatoryTransactionCode );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.bOneTransactionCodeOnly=" + acdEvent.eventInformation.callOutgoingDestinationData.bOneTransactionCodeOnly );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.callerId=" + acdEvent.eventInformation.callOutgoingDestinationData.callerId );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.callerNumber=" + acdEvent.eventInformation.callOutgoingDestinationData.callerNumber );
                    eventLogItem( "[" + i + "]callOutgoingDestinationData.callRecordingType=" + acdEvent.eventInformation.callOutgoingDestinationData.callRecordingType );
					eventLogItem( "[" + i + "]callOutgoingDestinationData.defaultTransactionCodesID=" + acdEvent.eventInformation.callOutgoingDestinationData.defaultTransactionCodesID );
					eventLogItem( "[" + i + "]callOutgoingDestinationData.userData=" + acdEvent.eventInformation.callOutgoingDestinationData.userData );
					eventLogItem( "[" + i + "]callOutgoingDestinationData.statisticsPartAID=" + acdEvent.eventInformation.callOutgoingDestinationData.statisticsPartAID );
					eventLogItem( "[" + i + "]callOutgoingDestinationData.testCall=" + acdEvent.eventInformation.callOutgoingDestinationData.testCall );
				}
				if( acdEvent.eventInformation.transferAnnouncementEndData != null )
                {
                    eventLogItem( "[" + i + "]transferAnnouncementEndData.userID=" + acdEvent.eventInformation.transferAnnouncementEndData.userID );
                    eventLogItem( "[" + i + "]transferAnnouncementEndData.callID=" + acdEvent.eventInformation.transferAnnouncementEndData.callID );
					eventLogItem( "[" + i + "]transferAnnouncementEndData.testCall=" + acdEvent.eventInformation.transferAnnouncementEndData.testCall );
                }
                if( acdEvent.eventInformation.transferAnnouncementStartData != null )
                {
                    eventLogItem( "[" + i + "]transferAnnouncementStartData.userID=" + acdEvent.eventInformation.transferAnnouncementStartData.userID );
                    eventLogItem( "[" + i + "]transferAnnouncementStartData.callID=" + acdEvent.eventInformation.transferAnnouncementStartData.callID );
					eventLogItem( "[" + i + "]transferAnnouncementStartData.testCall=" + acdEvent.eventInformation.transferAnnouncementStartData.testCall );
                }
                if( acdEvent.eventInformation.transferCallConnectData != null )
                {
                    eventLogItem( "[" + i + "]transferCallConnectData.userID=" + acdEvent.eventInformation.transferCallConnectData.userID );
                    eventLogItem( "[" + i + "]transferCallConnectData.callID=" + acdEvent.eventInformation.transferCallConnectData.callID );
					eventLogItem( "[" + i + "]transferCallConnectData.testCall=" + acdEvent.eventInformation.transferCallConnectData.testCall );
                }
                if( acdEvent.eventInformation.transferCallDisconnectData != null )
                {
                    eventLogItem( "[" + i + "]transferCallDisconnectData.userID=" + acdEvent.eventInformation.transferCallDisconnectData.userID );
                    eventLogItem( "[" + i + "]transferCallDisconnectData.callID=" + acdEvent.eventInformation.transferCallDisconnectData.callID );
					eventLogItem( "[" + i + "]transferCallDisconnectData.result=" + acdEvent.eventInformation.transferCallDisconnectData.result );
					eventLogItem( "[" + i + "]transferCallDisconnectData.testCall=" + acdEvent.eventInformation.transferCallDisconnectData.testCall );
                }
                if( acdEvent.eventInformation.transferCallEndData != null )
                {
                    eventLogItem( "[" + i + "]transferCallEndData.userID=" + acdEvent.eventInformation.transferCallEndData.userID );
                    eventLogItem( "[" + i + "]transferCallEndData.callID=" + acdEvent.eventInformation.transferCallEndData.callID );
					eventLogItem( "[" + i + "]transferCallEndData.testCall=" + acdEvent.eventInformation.transferCallEndData.testCall );
                }
                if( acdEvent.eventInformation.transferCallStartData != null )
                {
                    eventLogItem( "[" + i + "]transferCallStartData.userID=" + acdEvent.eventInformation.transferCallStartData.userID );
                    eventLogItem( "[" + i + "]transferCallStartData.callID=" + acdEvent.eventInformation.transferCallStartData.callID );
					eventLogItem( "[" + i + "]transferCallStartData.testCall=" + acdEvent.eventInformation.transferCallStartData.testCall );
                }
                if( acdEvent.eventInformation.transferCallWhisperingData != null )
                {
                    eventLogItem( "[" + i + "]transferCallWhisperingData.userID=" + acdEvent.eventInformation.transferCallWhisperingData.userID );
                    eventLogItem( "[" + i + "]transferCallWhisperingData.callID=" + acdEvent.eventInformation.transferCallWhisperingData.callID );
					eventLogItem( "[" + i + "]transferCallWhisperingData.testCall=" + acdEvent.eventInformation.transferCallWhisperingData.testCall );
                }
                if( acdEvent.eventInformation.callHoldData != null )
                {
                    eventLogItem( "[" + i + "]callHoldData.userID=" + acdEvent.eventInformation.callHoldData.userID );
                    eventLogItem( "[" + i + "]callHoldData.callID=" + acdEvent.eventInformation.callHoldData.callID );
					eventLogItem( "[" + i + "]callHoldData.testCall=" + acdEvent.eventInformation.callHoldData.testCall );
                }
                if( acdEvent.eventInformation.callRetrieveData != null )
                {
                    eventLogItem( "[" + i + "]callRetrieveData.userID=" + acdEvent.eventInformation.callRetrieveData.userID );
                    eventLogItem( "[" + i + "]callRetrieveData.callID=" + acdEvent.eventInformation.callRetrieveData.callID );
					eventLogItem( "[" + i + "]callRetrieveData.testCall=" + acdEvent.eventInformation.callRetrieveData.testCall );
                }
                if( acdEvent.eventInformation.callRecordingData != null )
                {
                    eventLogItem( "[" + i + "]callRecordingData.userID=" + acdEvent.eventInformation.callRecordingData.userID );
                    eventLogItem( "[" + i + "]callRecordingData.status=" + acdEvent.eventInformation.callRecordingData.status );
                    eventLogItem( "[" + i + "]callRecordingData.callID=" + acdEvent.eventInformation.callRecordingData.callID );
					eventLogItem( "[" + i + "]callRecordingData.testCall=" + acdEvent.eventInformation.callRecordingData.testCall );
                }
				if( acdEvent.eventInformation.callTransferredData != null )
				{
					eventLogItem( "[" + i + "]callTransferredData.userID=" + acdEvent.eventInformation.callTransferredData.userID );
					eventLogItem( "[" + i + "]callTransferredData.callID=" + acdEvent.eventInformation.callTransferredData.callID );
					eventLogItem( "[" + i + "]callTransferredData.testCall=" + acdEvent.eventInformation.callTransferredData.testCall );
				}
				if( acdEvent.eventInformation.webMessageData != null )
                {
                    eventLogItem( "[" + i + "]webMessageData.userID=" + acdEvent.eventInformation.webMessageData.userID );
                    eventLogItem( "[" + i + "]webMessageData.id=" + acdEvent.eventInformation.webMessageData.id );
                    eventLogItem( "[" + i + "]webMessageData.status=" + acdEvent.eventInformation.webMessageData.status );
                }
                if( acdEvent.eventInformation.mediaEventData != null )
                {
                    eventLogItem( "[" + i + "]mediaEventData.userID=" + acdEvent.eventInformation.mediaEventData.userID );
                    eventLogItem( "[" + i + "]mediaEventData.id=" + acdEvent.eventInformation.mediaEventData.mediaEventID + "/" + acdEvent.eventInformation.mediaEventData.type );
                    eventLogItem( "[" + i + "]mediaEventData.status=" + acdEvent.eventInformation.mediaEventData.status );
                    try
                    {
                        PrintmediaEventsGetAgent2( acdEvent.eventInformation.mediaEventData.userID, acdEvent.eventInformation.mediaEventData.mediaEventID );
                    }
                    catch( Exception e )
                    {
                        logAdd( "error=" + e.Message );
                    }
                }
                if( acdEvent.eventInformation.agentCountChangedData != null )
                {
                    eventLogItem( "[" + i + "]agentCountChangedData.acdGroupID=" + acdEvent.eventInformation.agentCountChangedData.acdGroupID );
                    eventLogItem( "[" + i + "]agentCountChangedData.agentCount=" + acdEvent.eventInformation.agentCountChangedData.agentCount );
                    eventLogItem( "[" + i + "]agentCountChangedData.acdGroupName=" + acdEvent.eventInformation.agentCountChangedData.acdGroupName );
                }
                if( acdEvent.eventInformation.agentStatusChangedData != null )
                {
                    eventLogItem( "[" + i + "]agentStatusChangedData.userID=" + acdEvent.eventInformation.agentStatusChangedData.userID );
                    eventLogItem( "[" + i + "]agentStatusChangedData.clientID=" + acdEvent.eventInformation.agentStatusChangedData.clientID );
                    eventLogItem( "[" + i + "]agentStatusChangedData.agentStatusID=" + acdEvent.eventInformation.agentStatusChangedData.agentStatusID );
                    eventLogItem( "[" + i + "]agentStatusChangedData.dtChanged=" + acdEvent.eventInformation.agentStatusChangedData.dtChanged );
                }
				if( acdEvent.eventInformation.customProcessInformationData != null ) 
				{
					eventLogItem( "[" + i + "]customProcessInformationData.callID=" + acdEvent.eventInformation.customProcessInformationData.callID );
					eventLogItem( "[" + i + "]customProcessInformationData.userID=" + acdEvent.eventInformation.customProcessInformationData.userID );
					eventLogItem( "[" + i + "]customProcessInformationData.information=" + acdEvent.eventInformation.customProcessInformationData.information );
					eventLogItem( "[" + i + "]customProcessInformationData.status=" + acdEvent.eventInformation.customProcessInformationData.status );
				}
				if( acdEvent.eventInformation.callCreateOutboundIVRData != null ) 
				{
					eventLogItem( "[" + i + "]callCreateOutboundIVRData.callID=" + acdEvent.eventInformation.callCreateOutboundIVRData.callID );
					eventLogItem( "[" + i + "]callCreateOutboundIVRData.destinationNumber=" + acdEvent.eventInformation.callCreateOutboundIVRData.destinationNumber);
					eventLogItem( "[" + i + "]callCreateOutboundIVRData.serviceNumberID=" + acdEvent.eventInformation.callCreateOutboundIVRData.serviceNumberID);
					eventLogItem( "[" + i + "]callCreateOutboundIVRData.userData=" + acdEvent.eventInformation.callCreateOutboundIVRData.userData);
					eventLogItem( "[" + i + "]callCreateOutboundIVRData.connres=" + acdEvent.eventInformation.callCreateOutboundIVRData.connres);
				}
                i++;
            }
        }

        private void BtnGetGroupSynonyms_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetGroupSynonyms_ClickUnsafe( sender, e ) ); }
        private void BtnGetGroupSynonyms_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                comboBoxTransferSynonym.Items.Clear();

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                Int32 ACDGroupsID = ( Int32 ) item.Tag;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.SynonymData[] groupSynonyms;

                resultV3 = wsClient2.getGroupSynonyms( sessionIDV3, ACDGroupsID, getUsersIDParameter(), out groupSynonyms );

                logAdd( "GetGroupSynonyms()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( groupSynonyms != null )
                {
                    if( groupSynonyms.Length == 0 )
                        logAdd( "There is no Synonym for this ACD Group" );
                    else
                        for( int i = 0; i < groupSynonyms.Length; i++ )
                        {
                            logAdd( "Synonym ID: " + groupSynonyms[ i ].id
                                + ", Synonym: " + groupSynonyms[ i ].synonym );
                            logAdd( "   brand: " + groupSynonyms[ i ].brand
                                + ", number: " + groupSynonyms[ i ].number );
                            logAdd( "   provider: " + groupSynonyms[ i ].provider
                                + ", tarifId: " + groupSynonyms[ i ].tarifId );
                            logAdd( "   tarifWavesId: " + groupSynonyms[ i ].tarifWavesId
                                + ", tarifName: " + groupSynonyms[ i ].tarifName );
                            logAdd( "   tarifComments: " + groupSynonyms[ i ].tarifComments );

                            ComboBoxItem itemSynonyms = new ComboBoxItem();
                            itemSynonyms.Content = groupSynonyms[ i ].synonym;
                            itemSynonyms.Tag = groupSynonyms[ i ].id;
                            comboBoxTransferSynonym.Items.Add( itemSynonyms );
                        }
                }
            }
        }

        private void eventThreadProc( object parms )
        {
            string sessionID = ( string ) parms;
            bool bRunning = true;
            eventLogClear();
            eventLogAdd( "eventThreadProc starting ..." );
            String stateData = "";
            while( bRunning )
            {
                try
                {
                    ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                    ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                    ACDWebServiceV3.AcdEvent[] eventData;
                    while( bRunning )
                    {
                        var start = DateTime.Now;
                        resultV3 = wsClient2.getNextEvents( sessionIDV3, ref stateData, out eventData );
                        var timeSpent = (DateTime.Now - start).TotalMilliseconds;
                        try
                        {
                            if( timeSpent < 5000 )
                            {
                                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK && eventData.Length == 0 )
                                    eventLogItem("timeout error timeSpent=" + timeSpent);
                            }
                            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK && eventData.Length == 0 )
                                eventLogAdd( ":" );
                            else
                            {
                                eventLogAdd( "getNextEvents(" + sessionIDV3 + ")=" + resultV3.ToString() + getUsersIDParameterInfo() );
                                eventLogItem( "stateData=" + stateData );
                                logEventData( eventData );
                                System.Threading.Thread.Sleep( 1000 );
                            }
                            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.ERR_SESSION )
                                bRunning = false;
                        }
                        catch( System.TimeoutException )
                        {
                            eventLogAdd( "." );
                        }
                    }
                }
                catch( System.TimeoutException )
                {
                    eventLogAdd( "." );
                }
                catch( Exception e )
                {
                    eventLogAdd( "exception " + e.GetType() + " in events thread: " + e.Message );
                    bRunning = false;
                }
                Monitor.Enter( pollingThreadSync );
                if( pollingThreadQuit == true ) bRunning = false;
                Monitor.Exit( pollingThreadSync );
            }
            eventLogAdd( "eventThreadProc stopping ..." );
            eventThread = null;
        }

        private void stopEventThread()
        {
            Monitor.Enter( pollingThreadSync );
            pollingThreadQuit = true;
            Monitor.Exit( pollingThreadSync );

            System.Threading.Thread.Sleep( 1000 );
        }

        private void startEventThread( String sessionID )
        {
            stopEventThread();
            Monitor.Enter( pollingThreadSync );
            pollingThreadQuit = false;
            Monitor.Exit( pollingThreadSync );
            if( eventThread != null ) eventThread.Abort();
            eventThread = new Thread( new ParameterizedThreadStart( eventThreadProc ) );
            eventThread.Start( sessionID );
        }

        private void Window_Closing( object sender, System.ComponentModel.CancelEventArgs e )
        {
            try
            {
                stopEventThread();
                doLogout();
            }
            catch( Exception ) { }
        }

        private void populatedComboBoxListStatusACD( int acdAgentStatusIdRemove )
        {
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.AcdAgentStatus[] acdAgentStatusValues;

            resultV3 = wsClient2.getAgentStatusValues( sessionIDV3, getUsersIDParameter(), out acdAgentStatusValues );
            logAdd( "getAgentStatusValues()=" + resultV3.ToString() + getUsersIDParameterInfo() );

            comboBoxNewStatusIDAgent.Items.Clear();
            if( acdAgentStatusValues != null )
            {
                foreach( ACDWebServiceV3.AcdAgentStatus agentStatus in acdAgentStatusValues )
                {
                    logAdd( "id: " + agentStatus.id );
                    logAdd( "   backgroundColor: " + agentStatus.backgroundColor );
                    logAdd( "   callback: " + agentStatus.callback );
                    logAdd( "   calls: " + agentStatus.calls );
                    logAdd( "   distributeDuringCall: " + agentStatus.distributeDuringCall );
                    logAdd( "   email: " + agentStatus.email );
                    logAdd( "   fax: " + agentStatus.fax );
                    logAdd( "   fontColor: " + agentStatus.fontColor );
                    logAdd( "   inLocation: " + agentStatus.inLocation );
                    logAdd( "   loggedIn: " + agentStatus.loggedIn );
                    logAdd( "   manualPostCall: " + agentStatus.manualPostCall );
                    logAdd( "   name: " + agentStatus.name );
                    logAdd( "   outbound: " + agentStatus.outbound );
                    logAdd( "   pause: " + agentStatus.pause );
                    logAdd( "   settable: " + agentStatus.settable );
                    logAdd( "   sms: " + agentStatus.sms );
                    logAdd( "   voiceMail: " + agentStatus.voiceMail );

                    ComboBoxItem item = new ComboBoxItem();
                    item.Content = agentStatus.name;
                    item.Tag = agentStatus.id;
                    comboBoxNewStatusIDAgent.Items.Add( item );
                }
            }
        }

        private String getNameACDStatus( int idACDStatus )
        {
            String NameACDStatus = "Logged Off";

            if( idACDStatus == 2 )
                NameACDStatus = "Call and Media";
            else if( idACDStatus == 3 )
                NameACDStatus = "Break";
            else if( idACDStatus == 4 )
                NameACDStatus = "Automatically Logged Out";
            else if( idACDStatus == 5 )
                NameACDStatus = "Post Call";

            return NameACDStatus;
        }

        private void BtnGroupTransactionCodes_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGroupTransactionCodes_ClickUnsafe( sender, e ) ); }
        private void BtnGroupTransactionCodes_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                comboBoxTransactionCodes.Items.Clear();

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                int acdGroupID = ( Int32 ) item.Tag;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.TransactionCodeData[] transactionCodes;

                resultV3 = wsClient2.getGroupTransactionCodes( sessionIDV3, acdGroupID, getUsersIDParameter(), out transactionCodes );

                logAdd( "getGroupTransactionCodes()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                if( transactionCodes != null )
                {
                    if( transactionCodes.Length == 0 )
                        logAdd( "There is no transactionCodes for this ACD Group" );
                    else
                        for( int i = 0; i < transactionCodes.Length; i++ )
                        {
                            logAdd( "Transaction Code ID: " + transactionCodes[ i ].id + ", Name: " + transactionCodes[ i ].name );
                            logAdd( "    displayOrder: " + transactionCodes[ i ].displayOrder + ", active: " + transactionCodes[ i ].active );
                            logAdd( "    name: " + transactionCodes[ i ].name + ", tarifComments: " + transactionCodes[ i ].tarifComments );
                            logAdd( "    tarifID: " + transactionCodes[ i ].tarifID + ", tarifComments: " + transactionCodes[ i ].tarifName );
                            logAdd( "    tarifWavesID: " + transactionCodes[ i ].tarifWavesID );
                            ComboBoxItem itemTransactionCode = new ComboBoxItem();
                            itemTransactionCode.Content = transactionCodes[ i ].name;
                            itemTransactionCode.Tag = transactionCodes[ i ].id;
                            comboBoxTransactionCodes.Items.Add( itemTransactionCode );
                        }
                }
            }
        }

        private void BtnSetTransactionCode_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnSetTransactionCode_ClickUnlock( sender, e ) ); }
        private void BtnSetTransactionCode_ClickUnlock( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" ) //any user is logged in
                logAdd( "Not logged in" );
            else if( textBoxCallID.Text == "" )
                logAdd( "Call ID required" );
            else if( comboBoxTransactionCodes.SelectedItem == null )
                logAdd( "Select a Call Marker" );
            else
            {
                ComboBoxItem item = ( ComboBoxItem ) comboBoxTransactionCodes.SelectedItem;
                Int32 callMarkerID = ( Int32 ) item.Tag;
                String nameCallMarker = ( String ) item.Content;

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.callTransactionCodeSet( sessionIDV3, textBoxCallID.Text, callMarkerID, getUsersIDParameter() );

                logAdd( "callTransactionCodeSet()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "ID Call=" + textBoxCallID.Text );
                logAdd( "TransactionCode ID=" + callMarkerID );
                logAdd( "TransactionCode Name=" + nameCallMarker );
            }

        }

        private void setDefaultValuesAndClearAllFields()
        {
            textBoxNewPasswordAgentPIN.Clear();
            textBoxNewUserPassword.Clear();
            textBoxSessionUUID.Clear();
            textGlobalACDStatisticsHourFrom.Text = "0";
            textGlobalACDStatisticsMinutesFrom.Text = "0";
            textGlobalACDStatisticsHourFromGroup.Text = "0";
            textGlobalACDStatisticsMinutesFromGroup.Text = "0";
            textBoxOldUserPassword.Clear();
            textBoxOldPasswordAgentPIN.Clear();
            textBoxCallID.Clear();
            textBoxCurrentStatusAgent.Clear();
            textBoxComment.Clear();
            textBoxInterval.Clear();
            textBoxPhoneNumberAgentStatus.Clear();
            comboAcdGroups.Items.Clear();
            comboBoxTransferSynonym.Items.Clear();
            comboBoxATransferACDGroup.Items.Clear();
            comboBoxTransactionCodes.Items.Clear();
            comboBoxNewStatusIDAgent.Items.Clear();
            checkBoxAnnouncement.IsChecked = true;

        }

        private void textBoxUsersUID_TextChanged( object sender, TextChangedEventArgs e )
        {
            userBackgroundLogo.Text = textBoxUsersUID.Text;
            //			passwordBoxPassword.Clear();
        }

        private void passwordBoxPassword_GotFocus( object sender, RoutedEventArgs e )
        {
            passwordBoxPassword.Clear();
        }

        private void BtnGetCallHistory_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetCallHistory_ClickUnsafe( sender, e ) ); }
        private void BtnGetCallHistory_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textBoxNumberCalls.Text == "" )
                logAdd( "Number Calls required" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.CallHistoryData[] callHistoryData;

                //TODO add the following params in gui
                int firstCall = 0;
                int numberCalls = 10;
                int totalNumberCalls = -1;

                Int32.TryParse( textBoxNumberCalls.Text, out numberCalls );
                Int32.TryParse( textBoxFirstCall.Text, out firstCall );

                resultV3 = wsClient2.getCallHistoryData( sessionIDV3, firstCall, numberCalls, getUsersIDParameter(), out totalNumberCalls, out callHistoryData );

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    logAdd( "GetCallHistory() == OK" );
                    logAdd( "totalNumberCalls: " + totalNumberCalls );

                    if( callHistoryData.Length == 0 )
                        logAdd( "There is no call history" );
                    else
                        for( int i = 0; i < callHistoryData.Length; i++ )
                        {
                            int numberCall = i + 1;
                            logAdd( "Statistics for Call " + numberCall + ": id:" + callHistoryData[ i ].id );
                            logAdd( "    acdGroupID: " + callHistoryData[ i ].acdGroupID );
                            logAdd( "    bMandatoryTransactionCode: " + callHistoryData[ i ].bMandatoryTransactionCode );
                            logAdd( "    bOneTransactionCodeOnly: " + callHistoryData[ i ].bOneTransactionCodeOnly );
                            logAdd( "    bOutbound: " + callHistoryData[ i ].bOutbound );
                            logAdd( "    callerNumber: " + callHistoryData[ i ].callerNumber );
                            logAdd( "    dtCallStart: " + callHistoryData[ i ].dtCallStart );
                            logAdd( "    dtCallEnd: " + callHistoryData[ i ].dtCallEnd );
                            logAdd( "    serviceNumber: " + callHistoryData[ i ].serviceNumber );
                            if( callHistoryData[ i ].transactionCodeIDs != null )
                            {
                                if( callHistoryData[ i ].transactionCodeIDs.Length > 0 )
                                {
                                    logAdd( "    transactionCodeIDs: " + string.Join( ",", callHistoryData[ i ].transactionCodeIDs ) );
                                }
                                else
                                {
                                    logAdd( "    transactionCodeIDs: int[0]" );
                                }
                            }
                            else
                            {
                                logAdd( "    transactionCodeIDs: null" );
                            }
                        }
                }
                else
                {
                    logAdd( "GetCallHistory()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                }
            }
        }

        private void BtnLoginACDGroup_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnLoginACDGroup_ClickUnsafe( sender, e ) ); }
        private void BtnLoginACDGroup_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {

                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                Int32 ACDGroupsID = ( Int32 ) item.Tag;

                resultV3 = wsClient2.loginACDGroup( sessionIDV3, ACDGroupsID, getUsersIDParameter() );

                if( resultV3.Equals( ACDWebServiceV3.NACD_RESPONSE_CODE.OK ) )
                {
                    logAdd( "LoginACDGroup() == OK" );
                    logAdd( "Login made in ACD Group \"" + item.Content.ToString() + "\"" );
                }
                else
                {
                    logAdd( "LoginACDGroup()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                    logAdd( "Not allowed to log in this ACD Group" );
                }
            }
        }

        private void BtnLogoutACDGroup_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnLogoutACDGroup_ClickUnsafe( sender, e ) ); }
        private void BtnLogoutACDGroup_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                Int32 ACDGroupsID = ( Int32 ) item.Tag;

                resultV3 = wsClient2.logoutACDGroup( sessionIDV3, ACDGroupsID, getUsersIDParameter() );

                if( resultV3.Equals( ACDWebServiceV3.NACD_RESPONSE_CODE.OK ) )
                {
                    logAdd( "LogoutACDGroup() == OK" );
                    logAdd( "Logout made from ACD Group \"" + item.Content.ToString() + "\"" );
                }
                else
                {
                    logAdd( "LogoutACDGroup()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                    logAdd( "Not allowed to log out from ACD Group" );
                }
            }
        }

        private void BtnGetACDDataActual_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetACDDataActual_ClickUnsafe( sender, e ) ); }
        private void BtnGetACDDataActual_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AcdGroupStatistics acdGroupStatistics;

                int timeFromHours = 0;
                int timeFromMinutes = 0;

                Int32.TryParse( textGlobalACDStatisticsHourFromGroup.Text.Trim(), out timeFromHours );
                Int32.TryParse( textGlobalACDStatisticsMinutesFromGroup.Text.Trim(), out timeFromMinutes );

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                Int32 ACDGroupsID = ( Int32 ) item.Tag;


                resultV3 = wsClient2.getGroupStatistics( sessionIDV3, ACDGroupsID, timeFromHours, timeFromMinutes, getUsersIDParameter(), out acdGroupStatistics );

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    logAdd( "getGroupStatistics (" + timeFromHours + ":" + timeFromMinutes + ") == OK" );
                    logAdd( "Number of Agents: " + acdGroupStatistics.numberAgents );
                    logAdd( "Actual Calls: " + acdGroupStatistics.actualCalls );
                    logAdd( "In Queue Calls: " + acdGroupStatistics.inQueueCalls );
                    logAdd( "Service Level: " + acdGroupStatistics.serviceLevel );
                    logAdd( "Average Total Duration: " + acdGroupStatistics.averageTotalDuration );
                    logAdd( "Average Total Waiting Time: " + acdGroupStatistics.averageTotalWaitingTime );
                }
                else
                {
                    logAdd( "getGroupStatistics()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                }
            }
        }

        private void BtnGetOverAllACDData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetOverAllACDData_ClickUnsafe( sender, e ) ); }
        private void BtnGetOverAllACDData_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textGlobalACDStatisticsHourFrom.Text == "" )
                logAdd( "Time required" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.GlobalACDStatistics globalACDStatistics;

                int timeFromHours = 0;
                int timeFromMinutes = 0;

                Int32.TryParse( textGlobalACDStatisticsHourFrom.Text.Trim(), out timeFromHours );
                Int32.TryParse( textGlobalACDStatisticsMinutesFrom.Text.Trim(), out timeFromMinutes );

                resultV3 = wsClient2.getGlobalACDStatistics( sessionIDV3, timeFromHours, timeFromMinutes, getUsersIDParameter(), out globalACDStatistics );

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    logAdd( "getGlobalACDStatistics (" + timeFromHours + ":" + timeFromMinutes + ") == OK" );
                    logAdd( "Total Calls: " + globalACDStatistics.totalCalls );
                    logAdd( "Answered Calls: " + globalACDStatistics.answeredCalls );
                    logAdd( "Missed Calls: " + globalACDStatistics.missedCalls );
                    logAdd( "Availability: " + globalACDStatistics.availability );
                    logAdd( "In Service: " + globalACDStatistics.inService );
                    logAdd( "Max Waiting Time: " + globalACDStatistics.maxWaitingTime );
                    logAdd( "Queued Calls: " + globalACDStatistics.queuedCalls );
                }
                else
                {
                    logAdd( "getGlobalACDStatistics()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                }
            }
        }

        private void BtnChangeAgentPIN_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnChangeAgentPIN_ClickUnsafe( sender, e ) ); }
        private void BtnChangeAgentPIN_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textBoxNewPasswordAgentPIN.Text == "" )
                logAdd( "New AgentPIN missing" );
            else
            {
                ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDClientTest.ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                String oldPin = textBoxOldPasswordAgentPIN.Text;
                String newPin = textBoxNewPasswordAgentPIN.Text;

                resultV3 = wsClient2.setAgentPIN( sessionIDV3, oldPin, newPin, getUsersIDParameter() );
                logAdd( "ChangeAgentPIN()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }

        }

        private void BtnChangeUserPassword_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnChangeUserPassword_ClickUnsafe( sender, e ) ); }
        private void BtnChangeUserPassword_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( textBoxNewUserPassword.Text == "" )
                logAdd( "New User Password missing" );
            else if( textBoxOldUserPassword.Text == "" )
                logAdd( "Old User Password missing" );
            else
            {
                ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDClientTest.ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDClientTest.ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                String oldPassword = textBoxOldUserPassword.Text;
                String newPassword = textBoxNewUserPassword.Text;

                resultV3 = wsClient2.setUserPassword( sessionIDV3, oldPassword, newPassword, getUsersIDParameter() );
                logAdd( "setUserPassword()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }

        }

        private void BtnCallTransferAgent_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferAgent_ClickUsafe( sender, e ) ); }
        private void BtnCallTransferAgent_ClickUsafe( object sender, RoutedEventArgs e )
        {
            ComboBoxItem item = ( ComboBoxItem ) comboBoxCallTransferAgent.SelectedItem;
            Int32 AgentID = ( Int32 ) item.Tag;

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callTransferAgent( sessionIDV3, textBoxCallID.Text, AgentID, getUsersIDParameter() );

            logAdd( "callTransferAgent()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallTransferStop_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferStop_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferStop_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callTransferStop( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

            logAdd( "callTransferStop()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnAgentsforClients_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnAgentsforClients_ClickUnsafe( sender, e ) ); }
        private void BtnAgentsforClients_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.AgentData[] clientsAgents;

            resultV3 = wsClient2.getClientAgents( sessionIDV3, getUsersIDParameter(), out clientsAgents );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "getClientAgents() == OK" );

                if( clientsAgents.Length == 0 )
                    logAdd( "There are no Agents" );
                else
                    for( int i = 0; i < clientsAgents.Length; i++ )
                    {
                        logAdd( "[" + clientsAgents[ i ].id + "]" + clientsAgents[ i ].name + ", " + clientsAgents[ i ].firstName );
                        logAdd( "    acdAgentStatusId=" + clientsAgents[ i ].acdAgentStatusId + ", dtAcdStatusID=" + clientsAgents[ i ].dtAcdStatusID );
                        logAdd( "    number=" + clientsAgents[ i ].number + ", PIN=" + clientsAgents[ i ].PIN );
						logAdd( "    agentNumber=" + clientsAgents[ i ].agentNumber + ", UID=" + clientsAgents[ i ].UID );
                    
						ComboBoxItem itemAgent = new ComboBoxItem();
                        itemAgent.Content = clientsAgents[ i ].name + ", " + clientsAgents[ i ].firstName;
                        itemAgent.Tag = clientsAgents[ i ].id;
                        comboAcdUsersAdmin.Items.Add( itemAgent );
					}
            }
            else
                logAdd( "getClientAgents()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallOnHold_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallOnHold_ClickUnsafe( sender, e ) ); }
        private void BtnCallOnHold_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callHold( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

            logAdd( "callHold()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }
	
        private void BtnStartCustomProcess_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnStartCustomProcess_ClickUnsafe( sender, e ) ); }
        private void BtnStartCustomProcess_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.customProcessStart( sessionIDV3, textBoxCallID.Text, textBoxProcessIdentifier.Text, textBoxTransactionIdentifier.Text, getUsersIDParameter() );

            logAdd( "customProcessStart()=" + resultV3.ToString() + getUsersIDParameterInfo() );
		}


        private void BtnCallRetrieve_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallRetrieve_ClickUnsafe( sender, e ) ); }
        private void BtnCallRetrieve_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callRetrieve( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

            logAdd( "callRetrieve()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallRecrdingStart_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallRecrdingStart_ClickUnsafe( sender, e ) ); }
        private void BtnCallRecrdingStart_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            String subject = "";

            if( textBoxSubjectCallRecording.Text == "" )
                subject = "Default";
            else
                subject = textBoxSubjectCallRecording.Text;

            resultV3 = wsClient2.callRecordingStart( sessionIDV3, textBoxCallID.Text, textBoxSubjectCallRecording.Text, getUsersIDParameter() );

            logAdd( "callRecordingStart()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallRecrdingStop_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallRecrdingStop_ClickUnsafe( sender, e ) ); }
        private void BtnCallRecrdingStop_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callRecordingStop( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

            logAdd( "callRecordingStop()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnClearLogEvent_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnClearLogEvent_ClickUnsafe( sender, e ) ); }
        private void BtnClearLogEvent_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            eventLogClear();
        }

        private void BtnStopPostCall_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnStopPostCall_ClickUnsafe( sender, e ) ); }
        private void BtnStopPostCall_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.stopPostCall( sessionIDV3, getUsersIDParameter() );

                logAdd( "StopPostCall()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }
            textBoxInterval.Clear();
        }

        private void BtnUsersLoggedInAgents_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnUsersLoggedInAgents_ClickUnsafe( sender, e ) ); }
        private void BtnUsersLoggedInAgents_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            comboBoxCallTransferAgent.Items.Clear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.AgentData[] clientsAgents;

            resultV3 = wsClient2.getLoggedInAgents( sessionIDV3, getUsersIDParameter(), out clientsAgents );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "getLoggedInAgents() == OK" );

                if( clientsAgents.Length == 0 )
                    logAdd( "There are no Users as Agents" );
                else
                    for( int i = 0; i < clientsAgents.Length; i++ )
                    {
                        logAdd( "[" + clientsAgents[ i ].id + "]" + clientsAgents[ i ].name + ", " + clientsAgents[ i ].firstName );
                        logAdd( "    acdAgentStatusId=" + clientsAgents[ i ].acdAgentStatusId + ", dtAcdStatusID=" + clientsAgents[ i ].dtAcdStatusID );
                        logAdd( "    number=" + clientsAgents[ i ].number + ", PIN=" + clientsAgents[ i ].PIN );
								logAdd( "    agentNumber=" + clientsAgents[ i ].agentNumber + ", UID=" + clientsAgents[ i ].UID );

                        ComboBoxItem itemAgent = new ComboBoxItem();
                        itemAgent.Content = clientsAgents[ i ].name + ", " + clientsAgents[ i ].firstName;
                        itemAgent.Tag = clientsAgents[ i ].id;
                        comboBoxCallTransferAgent.Items.Add( itemAgent );
                    }
            }
            else
                logAdd( "getLoggedInAgents()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallTransferNumberAnnouncement_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferNumberAnnouncement_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferNumberAnnouncement_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            // variable to check if phone is a number
            bool phoneIsNumber = System.Text.RegularExpressions.Regex.IsMatch( textBoxPhoneNumberTransfer.Text, "^[0-9]+$" );

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                bool announcement = checkBoxAnnouncement.IsChecked.HasValue && ( checkBoxAnnouncement.IsChecked == true );
                resultV3 = wsClient2.callTransferNumberAnnouncement( sessionIDV3, textBoxCallID.Text, textBoxPhoneNumberTransfer.Text, announcement, getUsersIDParameter() );

                logClear();
                logAdd( "CallTransferNumberAnnouncement()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "Call ID=" + textBoxCallID.Text );
                logAdd( "Phone Number to transfer=" + textBoxPhoneNumberTransfer.Text );
                logAdd( "Announcement=" + checkBoxAnnouncement.ToString() );
            }
        }

        private void BtnCallTransferNumberTransfer_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferNumberTransfer_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferNumberTransfer_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            // variable to check if phone is a number
            bool phoneIsNumber = System.Text.RegularExpressions.Regex.IsMatch( textBoxPhoneNumberTransfer.Text, "^[0-9]+$" );

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.callTransferNumber( sessionIDV3, textBoxCallID.Text, textBoxPhoneNumberTransfer.Text, getUsersIDParameter() );

                logClear();
                logAdd( "callTransferNumber()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                logAdd( "Call ID=" + textBoxCallID.Text );
                logAdd( "Phone Number to transfer == " + textBoxPhoneNumberTransfer.Text );
            }
        }

        private void BtnJunkCall_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnJunkCall_ClickUnsafe( sender, e ) ); }
        private void BtnJunkCall_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

                resultV3 = wsClient2.callSetJunk( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    BtnJunkCall.IsEnabled = false;
                }

                logClear();
                logAdd( "callSetJunk()=" + resultV3.ToString() + getUsersIDParameterInfo() );
            }
        }

        private void textBoxCallID_TextChanged( object sender, TextChangedEventArgs e )
        {
            BtnJunkCall.IsEnabled = true;
        }

        private void Window_Loaded( object sender, RoutedEventArgs e )
        {
            textBoxResellersUID.Text = Properties.Settings.Default.DefaultReseller;
            textBoxClientsUID.Text = Properties.Settings.Default.DefaultClient;
            textBoxUsersUID.Text = GetInstancePart( Properties.Settings.Default.DefaultUser, App.InstanceNumber );
            passwordBoxPassword.Password = GetInstancePart( Properties.Settings.Default.DefaultPassword, App.InstanceNumber );
            setDefaultValuesAndClearAllFields();
            if( Properties.Settings.Default.AutoLogin )
                BtnLogin_Click( null, null );

            textBoxWebMessageFrom.Text = DateTime.Now.ToString();
            textBoxWebMessageTo.Text = DateTime.Now.ToString();
        }

        private string GetInstancePart( string pipedString, int instance )
        {
            if( instance < 0 ) return pipedString;
            string[] parts = pipedString.Split( '|' );
            if( parts.Length <= instance ) return pipedString;
            return parts[ instance ];
        }

        private void btnClickToCall_Click( object sender, RoutedEventArgs e ) { SafeClick( () => btnClickToCall_ClickUnsafe( sender, e ) ); }
        private void btnClickToCall_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            ComboBoxItem item = ( ComboBoxItem ) comboBoxClickToCallACDGroup.SelectedItem;
            Int32 AcdGroupsID = item == null ? 0 : ( Int32 ) item.Tag;

            logClear();
            logAdd( "Call Number=" + textBoxPhoneNumberOutbound.Text );
            logAdd( "Acd Groups ID=" + AcdGroupsID );

            int serviceNumberIDTemp = 0;
            System.Nullable<int> serviceNumberID = null;
            if( Int32.TryParse( textBoxServiceNumberIDClickToCall.Text, out serviceNumberIDTemp ) )
            {
                logAdd( "ServiceNumberID '" + textBoxServiceNumberIDClickToCall.Text + "' parsed OK" );
                serviceNumberID = serviceNumberIDTemp;
            }
            logAdd( "ServiceNumber ID=" + ( serviceNumberID.HasValue ? serviceNumberID.Value.ToString() : "null" ) );
            logAdd( "Caller Number=" + textBoxCallerNumber.Text );
            logAdd( "User Data value=" + textBoxUserData.Text );

            int timeout;
            if( !Int32.TryParse( textBoxTimeout.Text, out timeout ) )
            {
                logAdd( "Timeout value '" + textBoxTimeout.Text + "' is wrong" );
                return;
            }
            logAdd( "Timeout ID=" + timeout );

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			string userData = textBoxUserData.Text;
			if( userData == null ) 
			{
				userData = "";
			}
			else if( userData == "NULL" )
			{
				userData = null;
			}

			bool? bSuppressPreview = checkBoxSuppressPreview.IsChecked;

            resultV3 = wsClient2.callCreateOutbound( sessionIDV3, AcdGroupsID, serviceNumberID, textBoxCallerNumber.Text, textBoxPhoneNumberOutbound.Text, timeout, userData, getUsersIDParameter(), bSuppressPreview );

            logAdd( "callCreateOutbound()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void btnClickToCallIVR_Click( object sender, RoutedEventArgs e ) { SafeClick( () => btnClickToCallIVR_ClickUnsafe( sender, e ) ); }
        private void btnClickToCallIVR_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            logAdd( "Call Number=" + textBoxPhoneNumberOutbound.Text );

            int serviceNumberID = 0;
            if( Int32.TryParse( textBoxServiceNumberIDClickToCall.Text, out serviceNumberID ) )
            {
                logAdd( "ServiceNumberID '" + textBoxServiceNumberIDClickToCall.Text + "' parsed OK" );
            }
            logAdd( "ServiceNumber ID=" + serviceNumberID );
            logAdd( "User Data value=" + textBoxUserData.Text );

            int timeout;
            if( !Int32.TryParse( textBoxTimeout.Text, out timeout ) )
            {
                logAdd( "Timeout value '" + textBoxTimeout.Text + "' is wrong" );
                return;
            }
            logAdd( "Timeout ID=" + timeout );

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			string userData = textBoxUserData.Text;
			if( userData == null ) 
			{
				userData = "";
			}
			else if( userData == "NULL" )
			{
				userData = null;
			}

            resultV3 = wsClient2.callCreateOutboundIVR( sessionIDV3, null, serviceNumberID, textBoxPhoneNumberOutbound.Text, timeout, userData );

            logAdd( "callCreateOutbound()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnGetServiceNumbersForAgent_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetServiceNumbersForAgent_Click_ClickUnsafe( sender, e ) ); }
        private void BtnGetServiceNumbersForAgent_Click_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.ServiceNumber[] serviceNumbers;

            resultV3 = wsClient2.serviceNumbersGetAgent( sessionIDV3, getUsersIDParameter(), out serviceNumbers );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "serviceNumbersGetAgent() == OK" );

                if( serviceNumbers.Length == 0 )
                    logAdd( "There are no Users as Agents" );
                else
                    for( int i = 0; i < serviceNumbers.Length; i++ )
                    {
                        logAdd( "[" + serviceNumbers[ i ].id + "], name=" + serviceNumbers[ i ].name + ", number=" + serviceNumbers[ i ].number + ", name2=" + serviceNumbers[ i ].name2 );
                    }
            }
            else
                logAdd( "serviceNumbersGetAgent()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnSendSMS_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnSendSMS_Click_ClickUnsafe( sender, e ) ); }
        private void BtnSendSMS_Click_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int serviceNumberID;
            if( !Int32.TryParse( textBoxSMSServiceNumberID.Text.Trim(), out serviceNumberID ) )
            {
                logAdd( "ServiceNumberID '" + textBoxSMSServiceNumberID.Text + "' is wrong" );
                return;
            }
            logAdd( "ServiceNumber ID=" + serviceNumberID );
            logAdd( "Caller Number=" + textBoxSMSCallerNumber.Text );
            logAdd( "Destination Number=" + textBoxSMSDestinationNumber.Text );
            logAdd( "Content=" + textBoxSMSContent.Text );

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.sendSMS( sessionIDV3, serviceNumberID, textBoxSMSCallerNumber.Text, textBoxSMSDestinationNumber.Text, textBoxSMSContent.Text, getUsersIDParameter() );

            logAdd( "sendSMS()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnWebMessageCreate_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnWebMessageCreate_ClickUnsafe( sender, e ) ); }
        private void BtnWebMessageCreate_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            ACDWebServiceV3.WebMessage webMessage = new ACDWebServiceV3.WebMessage();
            int webMessageType;
            if( !Int32.TryParse( textBoxWebMessageType.Text, out webMessageType ) )
            {
                logAdd( "WebMessageType '" + textBoxWebMessageType.Text + "' is wrong" );
                return;
            }
            webMessage.type = webMessageType;
            logAdd( "acdAgentIDs=" + textBoxWebMessageAcdAgentIDs.Text );
            if( textBoxWebMessageAcdAgentIDs.Text.Length > 0 )
                webMessage.acdAgentIDs = textBoxWebMessageAcdAgentIDs.Text.Split( ',' ).Select( o => ( int )Convert.ToInt32( o ) ).ToList().ToArray();
            logAdd( "acdGroupIDs=" + textBoxWebMessageAcdGroupIDs.Text );
            if( textBoxWebMessageAcdGroupIDs.Text.Length > 0 )
                webMessage.acdGroupIDs = textBoxWebMessageAcdGroupIDs.Text.Split( ',' ).Select( o => ( int )Convert.ToInt32( o ) ).ToList().ToArray();
            logAdd( "IsActive=" + checkBoxWebMessageIsActive.IsChecked );
            webMessage.bActive = checkBoxWebMessageIsActive.IsChecked.Value;

            logAdd( "From=" + textBoxWebMessageFrom.Text );
            if( textBoxWebMessageFrom.Text.Length > 0 )
            {
                webMessage.dtFrom = Convert.ToDateTime( textBoxWebMessageFrom.Text );
            }

            logAdd( "To=" + textBoxWebMessageTo.Text );
            if( textBoxWebMessageTo.Text.Length > 0 )
            {
                webMessage.dtTo = Convert.ToDateTime( textBoxWebMessageTo.Text );
            }

            logAdd( "Subject=" + textBoxWebMessageSubject.Text );
            webMessage.subject = textBoxWebMessageSubject.Text;
            logAdd( "Message=" + textBoxWebMessageMessage.Text );
            webMessage.message = textBoxWebMessageMessage.Text;

            logAdd( "acdWebMessageID=" + textBoxWebMessageId );
            int webMessageId = 0;
            if( textBoxWebMessageId.Text.Length > 0 )
                if( !Int32.TryParse( textBoxWebMessageId.Text, out webMessageId ) )
                {
                    logAdd( "WebMessageId '" + textBoxWebMessageId.Text + "' is wrong" );
                    return;
                }
            webMessage.id = webMessageId;

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.webMessageCreate( sessionIDV3, webMessage, getUsersIDParameter() );

            logAdd( "webMessageCreate()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnWebMessageSetStatus_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnWebMessageSetStatus_ClickUnsafe( sender, e ) ); }
        private void BtnWebMessageSetStatus_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int webMessageID;
            if( !Int32.TryParse( textBoxWebMessageId.Text, out webMessageID ) )
            {
                logAdd( "WebMessageId '" + textBoxWebMessageId.Text + "' is wrong" );
                return;
            }
            logAdd( "webMessage ID=" + webMessageID );
            logAdd( "IsSeen=" + checkBoxWebMessageIsSeen.IsChecked );


            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.webMessageSetStatus( sessionIDV3, webMessageID, checkBoxWebMessageIsSeen.IsChecked.Value, getUsersIDParameter() );

            logAdd( "webMessageSetStatus()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnWebMessageGetAll_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnWebMessageGetAll_ClickUnsafe( sender, e ) ); }
        private void BtnWebMessageGetAll_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.WebMessage[] webMessages;

            resultV3 = wsClient2.webMessagesGetAll( sessionIDV3, getUsersIDParameter(), out webMessages );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "webMessagesGetAll() == OK" );

                if( webMessages.Length == 0 )
                    logAdd( "There are no Messages" );
                else
                    for( int i = 0; i < webMessages.Length; i++ )
                    {
                        logAdd( "[" + webMessages[ i ].id + "] type=" + webMessages[ i ].type );
                        logAdd( "    acdAgentIDs=" + webMessages[ i ].acdAgentIDs );
                        logAdd( "    acdGroupIDs=" + webMessages[ i ].acdGroupIDs );
                        logAdd( "    bActive=" + webMessages[ i ].bActive );
                        logAdd( "    dtFrom=" + webMessages[ i ].dtFrom );
                        logAdd( "    dtTo=" + webMessages[ i ].dtTo );
                        logAdd( "    dtModified=" + webMessages[ i ].dtModified );
                        logAdd( "    subject=" + webMessages[ i ].subject );
                        logAdd( "    message=" + webMessages[ i ].message );
                    }
            }
            else
                logAdd( "webMessagesGetAll()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnWebMessageDelete_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnWebMessageDelete_ClickUnsafe( sender, e ) ); }
        private void BtnWebMessageDelete_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int webMessageID;
            if( !Int32.TryParse( textBoxWebMessageId.Text, out webMessageID ) )
            {
                logAdd( "WebMessageId '" + textBoxWebMessageId.Text + "' is wrong" );
                return;
            }
            logAdd( "webMessage ID=" + webMessageID );

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.webMessageDelete( sessionIDV3, webMessageID, getUsersIDParameter() );

            logAdd( "webMessageDelete()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnFreeSeatingWhitelistData_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnFreeSeatingWhitelistData_ClickUnsafe( sender, e ) ); }
        private void BtnFreeSeatingWhitelistData_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.FreeSeatingWhitelistEntry[] freeSeatingWhitelistEntries;

            resultV3 = wsClient2.getFreeSeatingWhitelistData( sessionIDV3, getUsersIDParameter(), out freeSeatingWhitelistEntries );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "getFreeSeatingWhitelistData() == OK" );

                if( freeSeatingWhitelistEntries.Length == 0 )
                    logAdd( "There are no Entries" );
                else
                    for( int i = 0; i < freeSeatingWhitelistEntries.Length; i++ )
                    {
                        logAdd( "[" + freeSeatingWhitelistEntries[ i ].id + "] number=" + freeSeatingWhitelistEntries[ i ].number );
                        logAdd( "    comments=" + freeSeatingWhitelistEntries[ i ].comments );
                    }
            }
            else
                logAdd( "getFreeSeatingWhitelistData()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnSetSessionLanguage_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnSetSessionLanguage_ClickUnsafe( sender, e ) ); }
        private void BtnSetSessionLanguage_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int languageID;
            if( !Int32.TryParse( textBoxlanguageID.Text, out languageID ) )
            {
                logAdd( "WebMessageId '" + textBoxlanguageID.Text + "' is wrong" );
                return;
            }
            logAdd( "languageID=" + languageID );

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.setSessionLanguage( sessionIDV3, languageID, getUsersIDParameter() );

            logAdd( "setSessionLanguage()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnmediaEventsGetAgentHistory_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnmediaEventsGetAgentHistory_ClickUnsafe( sender, e ) ); }
        private void BtnmediaEventsGetAgentHistory_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int firstEvent = 0;
            int numberEvents = 0;
            int totalNumberEvents = 0;
            if( !Int32.TryParse( textBoxFirstEvent.Text, out firstEvent ) )
            {
                logAdd( "firstEvent is wrong" );
                return;
            }
            if( !Int32.TryParse( textBoxNumberEvents.Text, out numberEvents ) )
            {
                logAdd( "numberEvents is wrong" );
                return;
            }

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.MediaEvent[] mediaEvents;

            resultV3 = wsClient2.mediaEventsGetAgentHistory( sessionIDV3, firstEvent, numberEvents, getUsersIDParameter(), out totalNumberEvents, out mediaEvents );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "mediaEventsGetAgentHistory() == OK" );

                if( mediaEvents.Length == 0 )
                    logAdd( "There are no Entries" );
                else
                    for( int i = 0; i < mediaEvents.Length; i++ )
                    {
                        logAdd( "[" + mediaEvents[ i ].id + "] type=" + mediaEvents[ i ].type );
                        logAdd( "    acdGroupID=" + mediaEvents[ i ].acdGroupID );
                        logAdd( "    comment=" + mediaEvents[ i ].comment );
                        logAdd( "    dtCreated=" + mediaEvents[ i ].dtCreated );
                        logAdd( "    dtModified=" + mediaEvents[ i ].dtModified );
                        logAdd( "    fileNameURI=" + mediaEvents[ i ].fileNameURI );
                        logAdd( "    sender=" + mediaEvents[ i ].sender );
                        logAdd( "    status=" + mediaEvents[ i ].status );
                        logAdd( "    subject=" + mediaEvents[ i ].subject );
                        logAdd( "    receiver=" + mediaEvents[ i ].receiver );
                        logAdd( "    eventData=" + mediaEvents[ i ].eventData );
                    }
            }
            else
                logAdd( "mediaEventsGetAgentHistory()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnMediaEventUpdate_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnMediaEventUpdate_ClickUnsafe( sender, e ) ); }
        private void BtnMediaEventUpdate_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int mediaEventID = 0;
            int? acdAgentID = null; int acdAgentIDOut = 0;
            int? acdGroupID = null; int acdGroupIDOut = 0;
            if( !Int32.TryParse( textBoxMediaEventID.Text, out mediaEventID ) )
            {
                logAdd( "mediaEventID is wrong" );
                return;
            }
            if( Int32.TryParse( textBoxMediaEventUpdateAcdAgentID.Text, out acdAgentIDOut ) )
            {
                acdAgentID = acdAgentIDOut;
            }
            if( Int32.TryParse( textBoxMediaEventUpdateAcdGroupID.Text, out acdGroupIDOut ) )
            {
                acdGroupID = acdGroupIDOut;
            }

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            ACDWebServiceV3.MediaEventStatus status = ( ACDWebServiceV3.MediaEventStatus ) Enum.Parse( typeof( ACDWebServiceV3.MediaEventStatus ), ComboBoxMediaEventStatus.Text, true );
            ACDWebServiceV3.MediaEventFunction function = ( ACDWebServiceV3.MediaEventFunction ) Enum.Parse( typeof( ACDWebServiceV3.MediaEventFunction ), ComboBoxMediaEventFunction.Text, true );

            resultV3 = wsClient2.mediaEventUpdate( sessionIDV3, mediaEventID, function, acdAgentID, acdGroupID, textBoxMediaEventUpdateComment.Text, status, getUsersIDParameter() );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "mediaEventUpdate() == OK" );
            }
            else
                logAdd( "mediaEventUpdate()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnMediaEventGetHistory_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnMediaEventGetHistory_ClickUnsafe( sender, e ) ); }
        private void BtnMediaEventGetHistory_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            int mediaEventID = 0;
            if( !Int32.TryParse( textBoxMediaEventGetHistoryEventID.Text, out mediaEventID ) )
            {
                logAdd( "mediaEventID is wrong" );
                return;
            }
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDWebServiceV3.MediaEventHistory[] history;

            resultV3 = wsClient2.mediaEventGetHistory( sessionIDV3, mediaEventID, getUsersIDParameter(), out history );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "mediaEventGetHistory() == OK" );

                if( history.Length == 0 )
                    logAdd( "There are no Entries" );
                else
                    for( int i = 0; i < history.Length; i++ )
                    {
                        logAdd( "[" + i + "] statusNew=" + history[ i ].statusNew );
                        logAdd( "    statusOld=" + history[ i ].statusOld );
                        logAdd( "    comment=" + history[ i ].comment );
                        logAdd( "    dtTimestamp=" + history[ i ].dtTimestamp );
                        logAdd( "    user=" + history[ i ].user );
                    }
            }
            else
                logAdd( "mediaEventGetHistory()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnMediaEventsGetAgent_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnMediaEventsGetAgent_ClickUnsafe( sender, e ) ); }
        private void BtnMediaEventsGetAgent_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();
            PrintmediaEventsGetAgent( getUsersIDParameter() );
        }

        private void PrintmediaEventsGetAgent( int? usersID )
        {
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDClientTest.ACDWebServiceV3.MediaEvent[] mediaEvents;

            resultV3 = wsClient2.mediaEventsGetAgent( sessionIDV3, usersID, out mediaEvents );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "mediaEventsGetAgent() == OK" );

                if( mediaEvents.Length == 0 )
                    logAdd( "There are no Entries" );
                else
                    for( int i = 0; i < mediaEvents.Length; i++ )
                    {
                        logAdd( "[" + mediaEvents[ i ].id + "] acdGroupID=" + mediaEvents[ i ].acdGroupID );
                        logAdd( "    comment=" + mediaEvents[ i ].comment );
                        logAdd( "    dtCreated=" + mediaEvents[ i ].dtCreated );
                        logAdd( "    dtModified=" + mediaEvents[ i ].dtModified );
                        logAdd( "    fileNameURI=" + mediaEvents[ i ].fileNameURI );
                        logAdd( "    receiver=" + mediaEvents[ i ].receiver );
                        logAdd( "    sender=" + mediaEvents[ i ].sender );
                        logAdd( "    status=" + mediaEvents[ i ].status );
                        logAdd( "    subject=" + mediaEvents[ i ].subject );
                        logAdd( "    type=" + mediaEvents[ i ].type );
                        logAdd( "    eventData=" + mediaEvents[ i ].eventData );
                    }
            }
            else
                logAdd( "mediaEventsGetAgent()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void PrintmediaEventsGetAgent2( int? usersID, int eventsID )
        {
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
            ACDClientTest.ACDWebServiceV3.MediaEvent[] mediaEvents;

            resultV3 = wsClient2.mediaEventsGetAgent( sessionIDV3, usersID, out mediaEvents );

            if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
            {
                logAdd( "mediaEventsGetAgent() == OK" );

                if( mediaEvents.Length == 0 )
                    logAdd( "There are no Entries" );
                else
                {
                    StringBuilder buf = new StringBuilder();
                    for( int i = 0; i < mediaEvents.Length; i++ )
                    {
                        if( eventsID == mediaEvents[ i ].id )
                            buf.Append( "<" + mediaEvents[ i ].id + "> Group:" + mediaEvents[ i ].acdGroupID );
                        else
                            buf.Append( "[" + mediaEvents[ i ].id + "] Group:" + mediaEvents[ i ].acdGroupID );
                        buf.Append( "," + mediaEvents[ i ].dtCreated );
                        buf.Append( "," + mediaEvents[ i ].status );
                        buf.Append( "," + mediaEvents[ i ].type );
                        buf.Append( Environment.NewLine );
                    }
                    logAdd( buf.ToString() );
                }
            }
            else
                logAdd( "mediaEventsGetAgent()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnCallTransferTransfer_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallTransferTransfer_ClickUnsafe( sender, e ) ); }
        private void BtnCallTransferTransfer_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            resultV3 = wsClient2.callTransferTransfer( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

            logClear();
            logAdd( "Call ID=" + textBoxCallID.Text );
            logAdd( "callTransferTransfer()=" + resultV3.ToString() + getUsersIDParameterInfo() );
        }

        private void BtnFillComboboxes_Click( object sender, RoutedEventArgs e )
        {
            BtnAgentsforClients_Click( null, null );
            BtnGetAccountData_Click( null, null );
            BtnGetAgentGroupsData_Click( null, null );
            BtnGetAgentStatus_Click( null, null );
            BtnGetAllGroupsData_Click( null, null );
            BtnGetServiceNumbersForAgent_Click( null, null );
            BtnFreeSeatingWhitelistData_Click( null, null );
            BtnUsersLoggedInAgents_Click( null, null );
            logClear();
            logAdd( "Get Account Data" );
            logAdd( "Get Agent Groups Data" );
            logAdd( "Get Agent Status" );
            logAdd( "Get Agents for Clients" );
            logAdd( "Get Client Groups Data" );
            logAdd( "Free Seating Whitelist Data" );
            logAdd( "Get Service Numbers ForAgent" );
            logAdd( "Get Users Logged In Agents" );
        }

        private void BtnStartSecond_Click( object sender, RoutedEventArgs e )
        {
            string exeLocation = System.Reflection.Assembly.GetEntryAssembly().Location;
            Process process = new Process();
            process.StartInfo.FileName = exeLocation;
            process.StartInfo.Arguments = "second";
            process.StartInfo.WindowStyle = ProcessWindowStyle.Normal;
            process.Start();
        }

        internal void SetPosition( int count, int pos )
        {
            Thread worker = new Thread( delegate()
            {
                this.Dispatcher.Invoke(
                           DispatcherPriority.SystemIdle,
                           new Action( () =>
                           {
                               var instanceHeight = System.Windows.SystemParameters.WorkArea.Height / count;
                               Width = System.Windows.SystemParameters.WorkArea.Width;
                               Height = instanceHeight;
                               Left = System.Windows.SystemParameters.WorkArea.Left;
                               Top = System.Windows.SystemParameters.WorkArea.Top + pos * instanceHeight;
                               this.Activate();
                           }
                               ) );
            } );
            worker.Start();
        }

        private void BtnTile_Click( object sender, RoutedEventArgs e )
        {
            List<WindowServiceProxy> proxis = App.GetWindowsProxies();
            if( proxis.Count == 0 ) return;
            int i = 0;
            foreach( var p in proxis )
            {
                p.SetPosition( proxis.Count, i++ );
            }
        }

        private void BtnLoginAll_Click( object sender, RoutedEventArgs e )
        {
            List<WindowServiceProxy> proxis = App.GetWindowsProxies();
            if( proxis.Count == 0 ) return;
            foreach( var p in proxis )
            {
                p.Login();
            }
        }

        internal void ReLogin()
        {
            Thread worker = new Thread( delegate()
            {
                this.Dispatcher.Invoke(
                           DispatcherPriority.SystemIdle,
                           new Action( () =>
                           {
                               this.BtnLogout_Click( null, null );
                               this.BtnLogin_Click( null, null );
                           }
                               ) );
            } );
            worker.Start();
        }

        private void BtnCall2Dial_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCall2Dial_ClickUnsafe( sender, e ) ); }
        private void BtnCall2Dial_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

            String caller = textBoxCall2DialCaller.Text;
            String callerNumber = textBoxCall2DialCallerNumber.Text;
            String destinationNumber = textBoxCall2DialDestinationNumber.Text;
            int timeout = ( int ) Convert.ToInt32( textBoxCall2DialTimeout.Text );
            String userData = textBoxCall2DialUserData.Text;
            resultV3 = wsClient2.callCreateOutboundNoAgent( caller, callerNumber, destinationNumber, timeout, userData );

            logClear();
            logAdd( "caller=" + caller );
            logAdd( "callerNumber=" + callerNumber );
            logAdd( "destinationNumber=" + destinationNumber );
            logAdd( "timeout=" + timeout );
            logAdd( "userData=" + userData );
            logAdd( "callCreateOutboundNoAgent()=" + resultV3.ToString() + getUsersIDParameterInfo() );

        }

	private void BtnAcdGroupAdminCreateAgentGroupMapping_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnAcdGroupAdminCreateAgentGroupMapping_ClickUnsafe( sender, e ) ); }

		private void BtnAcdGroupAdminCreateAgentGroupMapping_ClickUnsafe( object sender, RoutedEventArgs e )
		{
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			ComboBoxItem item = ( ComboBoxItem ) comboAcdGroupsAdmin.SelectedItem;
            Int32 AcdGroupsID = ( Int32 ) item.Tag;

			item = ( ComboBoxItem ) comboAcdUsersAdmin.SelectedItem;
            Int32 UsersID = ( Int32 ) item.Tag;

			ACDWebServiceV3.AcdAgentGroupMappingType acdAgentGroupMappingType = ACDWebServiceV3.AcdAgentGroupMappingType.PERMANENT;
			item = ( ComboBoxItem ) comboMappingType.SelectedItem;
			switch( (String) item.Tag ) 
			{
				case "1":
					acdAgentGroupMappingType = ACDWebServiceV3.AcdAgentGroupMappingType.PERMANENT;
					break;
				case "2":
					acdAgentGroupMappingType = ACDWebServiceV3.AcdAgentGroupMappingType.RESERVE;
					break;
				case "3":
					acdAgentGroupMappingType = ACDWebServiceV3.AcdAgentGroupMappingType.READONLY;
					break;
				case "4":
					acdAgentGroupMappingType = ACDWebServiceV3.AcdAgentGroupMappingType.INVISIBLE;
					break;
			}

			bool? bSupervisor = checkBoxSupervisor.IsChecked;
			bool? bMinLoggedInAgentRestrictionActive = checkBoxMinLoggedInAgentRestrictionActive.IsChecked;
			bool? bAdministrator = checkBoxAdministrator.IsChecked;
			
			item = ( ComboBoxItem ) comboSkillLevel.SelectedItem;
			int? agentSkill = null; int agentSkillOut = 0;
            if( Int32.TryParse( (String) item.Tag, out agentSkillOut ) )
            {
                agentSkill = agentSkillOut;
            }			

			bool? bLogoutDisabled = checkBoxLogoutDisabled.IsChecked;
			bool? bAutoDailyGroupLogout = checkBoxDailyGroupLogout.IsChecked;
			bool? bAutomaticLogoutCalls = checkBoxAutomaticLogoutCalls.IsChecked;

			resultV3 = wsClient2.acdGroupAdminCreateAgentGroupMapping( sessionIDV3, AcdGroupsID, UsersID, acdAgentGroupMappingType, (bool) bSupervisor, (bool) bMinLoggedInAgentRestrictionActive, (bool) bAdministrator, (int) agentSkill, (bool) bLogoutDisabled, (bool) bAutoDailyGroupLogout, (bool) bAutomaticLogoutCalls, false );
		
            logClear();
			logAdd( "AcdGroupsID=" + AcdGroupsID );
			logAdd( "UsersID=" + UsersID );
			logAdd( "bSupervisor=" + bSupervisor );
			logAdd( "bMinLoggedInAgentRestrictionActive=" + bMinLoggedInAgentRestrictionActive );
			logAdd( "bAdministrator=" + bAdministrator );
			logAdd( "agentSkill=" + agentSkill );
			logAdd( "bLogoutDisabled=" + bLogoutDisabled );
			logAdd( "bAutoDailyGroupLogout=" + bAutoDailyGroupLogout );
			logAdd( "bAutomaticLogoutCalls=" + bAutomaticLogoutCalls );

			logAdd( "acdGroupAdminCreateAgentGroupMapping()=" + resultV3.ToString() );
		}

		private void BtnAcdGroupAdminDeleteAgentGroupMapping_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnAcdGroupAdminDeleteAgentGroupMapping_ClickUnsafe( sender, e ) ); }

		private void BtnAcdGroupAdminDeleteAgentGroupMapping_ClickUnsafe( object sender, RoutedEventArgs e )
		{
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			ComboBoxItem item = ( ComboBoxItem ) comboAcdGroupsAdmin.SelectedItem;
            Int32 AcdGroupsID = ( Int32 ) item.Tag;

			item = ( ComboBoxItem ) comboAcdUsersAdmin.SelectedItem;
            Int32 UsersID = ( Int32 ) item.Tag;

			resultV3 = wsClient2.acdGroupAdminDeleteAgentGroupMapping( sessionIDV3, AcdGroupsID, UsersID );
		
            logClear();
			logAdd( "AcdGroupsID=" + AcdGroupsID );
			logAdd( "UsersID=" + UsersID );

			logAdd( "acdGroupAdminDeleteAgentGroupMapping()=" + resultV3.ToString() );
		}

		private void BtnAcdGroupAdminSetAgentGroupSkill_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnAcdGroupAdminSetAgentGroupSkill_ClickUnsafe( sender, e ) ); }

		private void BtnAcdGroupAdminSetAgentGroupSkill_ClickUnsafe( object sender, RoutedEventArgs e )
		{
            ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
            ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			ComboBoxItem item = ( ComboBoxItem ) comboAcdGroupsAdmin.SelectedItem;
            Int32 AcdGroupsID = ( Int32 ) item.Tag;

			item = ( ComboBoxItem ) comboAcdUsersAdmin.SelectedItem;
            Int32 UsersID = ( Int32 ) item.Tag;
			
			item = ( ComboBoxItem ) comboSkillLevel.SelectedItem;
			int? agentSkill = null; int agentSkillOut = 0;
            if( Int32.TryParse( (String) item.Tag, out agentSkillOut ) )
            {
                agentSkill = agentSkillOut;
            }			

			resultV3 = wsClient2.acdGroupAdminSetAgentGroupSkill( sessionIDV3, AcdGroupsID, UsersID, (int) agentSkill );
		
            logClear();
			logAdd( "AcdGroupsID=" + AcdGroupsID );
			logAdd( "UsersID=" + UsersID );
			logAdd( "agentSkill=" + agentSkill );

			logAdd( "acdGroupAdminSetAgentGroupSkill()=" + resultV3.ToString() );
		}

        private void BtnGetACDDataActual2_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnGetACDDataActual2_ClickUnsafe( sender, e ) ); }
        private void BtnGetACDDataActual2_ClickUnsafe( object sender, RoutedEventArgs e )
        {
            logClear();

            if( sessionIDV3 == "" )
                logAdd( "Not logged in" );
            else if( comboAcdGroups.SelectedItem == null )
                logAdd( "Select an ACD Group" );
            else
            {
                ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
                ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;
                ACDWebServiceV3.AcdGroupStatistics2[] acdGroupStatistics;

                int timeFromHours = 0;
                int timeFromMinutes = 0;

                Int32.TryParse( textGlobalACDStatisticsHourFromGroup.Text.Trim(), out timeFromHours );
                Int32.TryParse( textGlobalACDStatisticsMinutesFromGroup.Text.Trim(), out timeFromMinutes );

                ComboBoxItem item = ( ComboBoxItem ) comboAcdGroups.SelectedItem;
                Int32 ACDGroupsID = ( Int32 ) item.Tag;


                resultV3 = wsClient2.getGroupStatistics2( sessionIDV3, new int[] { ACDGroupsID }, timeFromHours, timeFromMinutes, getUsersIDParameter(), out acdGroupStatistics );

                if( resultV3 == ACDWebServiceV3.NACD_RESPONSE_CODE.OK )
                {
                    logAdd( "getGroupStatistics2 (" + timeFromHours + ":" + timeFromMinutes + ") == OK" );
                    foreach( var groupData in acdGroupStatistics )
                    {
                        logAdd( "" );
                        logAdd( "averageTotalDurationToday:[" + groupData.acdGroupID + "]: " + groupData.averageTotalDurationToday );
                        logAdd( "averageTotalWaitingTimeToday[" + groupData.acdGroupID + "]: " + groupData.averageTotalWaitingTimeToday );
                        logAdd( "currentAgents[" + groupData.acdGroupID + "]: " + groupData.currentAgents );
                        logAdd( "currentCalls[" + groupData.acdGroupID + "]: " + groupData.currentCalls );
                        logAdd( "currentInQueueCalls[" + groupData.acdGroupID + "]: " + groupData.currentInQueueCalls );
                        logAdd( "serviceLevelToday[" + groupData.acdGroupID + "]: " + groupData.serviceLevelToday );
                        logAdd( "totalAgentCallsOpen[" + groupData.acdGroupID + "]: " + groupData.totalAgentCallsOpen );
                        logAdd( "totalAgentCallsToday[" + groupData.acdGroupID + "]: " + groupData.totalAgentCallsToday );
                        logAdd( "totalCallBackOpen[" + groupData.acdGroupID + "]: " + groupData.totalCallBackOpen );
                        logAdd( "totalCallBackToday[" + groupData.acdGroupID + "]: " + groupData.totalCallBackToday );
                        logAdd( "totalCallsOpen[" + groupData.acdGroupID + "]: " + groupData.totalCallsOpen );
                        logAdd( "totalCallsToday[" + groupData.acdGroupID + "]: " + groupData.totalCallsToday );
                        logAdd( "totalExternalDestinationOpen[" + groupData.acdGroupID + "]: " + groupData.totalExternalDestinationOpen );
                        logAdd( "totalExternalDestinationToday[" + groupData.acdGroupID + "]: " + groupData.totalExternalDestinationToday );
                        logAdd( "totalHangupsOpen[" + groupData.acdGroupID + "]: " + groupData.totalHangupsOpen );
                        logAdd( "totalHangupsToday[" + groupData.acdGroupID + "]: " + groupData.totalHangupsToday );
                        logAdd( "totalOtherCallsOpen[" + groupData.acdGroupID + "]: " + groupData.totalOtherCallsOpen );
                        logAdd( "totalOtherCallsToday[" + groupData.acdGroupID + "]: " + groupData.totalOtherCallsToday );
                        logAdd( "totalVoiceMailOpen[" + groupData.acdGroupID + "]: " + groupData.totalVoiceMailOpen );
                        logAdd( "totalVoiceMailToday[" + groupData.acdGroupID + "]: " + groupData.totalVoiceMailToday );
                    }
                }
                else
                {
                    logAdd( "getGroupStatistics()=" + resultV3.ToString() + getUsersIDParameterInfo() );
                }
            }
        }

		private void BtnCallAnswer_Click( object sender, RoutedEventArgs e ) { SafeClick( () => BtnCallAnswer_ClickUnsafe( sender, e ) ); }
		private void BtnCallAnswer_ClickUnsafe( object sender, RoutedEventArgs e )
		{
			logClear();

			ACDWebServiceV3.AcdAgentClientServiceV3Client wsClient2 = new ACDWebServiceV3.AcdAgentClientServiceV3Client();
			ACDWebServiceV3.NACD_RESPONSE_CODE resultV3;

			resultV3 = wsClient2.callAnswer( sessionIDV3, textBoxCallID.Text, getUsersIDParameter() );

			logAdd( "callAnswer()=" + resultV3.ToString() + getUsersIDParameterInfo() );
		}
	}
}
